@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -83,7 +83,7 @@ public abstract class Prepared {
      */
     public Prepared(SessionLocal session) {
         this.session = session;
-        modificationMetaId = session.getDatabase().getModificationMetaId();
+        modificationMetaId = getDatabase().getModificationMetaId();
     }
 
     /**
@@ -124,7 +124,7 @@ public abstract class Prepared {
      * @return true if it must
      */
     public boolean needRecompile() {
-        Database db = session.getDatabase();
+        Database db = getDatabase();
         if (db == null) {
             throw DbException.get(ErrorCode.CONNECTION_BROKEN_1, "database closed");
         }
@@ -307,7 +307,7 @@ public abstract class Prepared {
     protected int getObjectId() {
         int id = persistedObjectId;
         if (id == 0) {
-            id = session.getDatabase().allocateObjectId();
+            id = getDatabase().allocateObjectId();
         } else if (id < 0) {
             throw DbException.getInternalError("Prepared.getObjectId() was called before");
         }
@@ -360,11 +360,11 @@ public abstract class Prepared {
     /**
      * Print information about the statement executed if info trace level is
      * enabled.
-     *
+     * @param database to update statistics
      * @param startTimeNanos when the statement was started
      * @param rowCount the query or update row count
      */
-    void trace(long startTimeNanos, long rowCount) {
+    void trace(Database database, long startTimeNanos, long rowCount) {
         if (session.getTrace().isInfoEnabled() && startTimeNanos > 0) {
             long deltaTimeNanos = System.nanoTime() - startTimeNanos;
             String params = Trace.formatParams(parameters);
@@ -372,9 +372,9 @@ public abstract class Prepared {
         }
         // startTime_nanos can be zero for the command that actually turns on
         // statistics
-        if (session.getDatabase().getQueryStatistics() && startTimeNanos != 0) {
+        if (database != null && database.getQueryStatistics() && startTimeNanos != 0) {
             long deltaTimeNanos = System.nanoTime() - startTimeNanos;
-            session.getDatabase().getQueryStatisticsData().update(toString(), deltaTimeNanos, rowCount);
+            database.getQueryStatisticsData().update(toString(), deltaTimeNanos, rowCount);
         }
     }
 
@@ -415,7 +415,7 @@ public abstract class Prepared {
      */
     private void setProgress() {
         if ((currentRowNumber & 127) == 0) {
-            session.getDatabase().setProgress(DatabaseEventListener.STATE_STATEMENT_PROGRESS, sqlStatement,
+            getDatabase().setProgress(DatabaseEventListener.STATE_STATEMENT_PROGRESS, sqlStatement,
                     currentRowNumber, 0L);
         }
     }
@@ -448,7 +448,7 @@ public abstract class Prepared {
      * @param values the values of the row
      * @return the exception
      */
-    protected DbException setRow(DbException e, long rowId, String values) {
+    protected final DbException setRow(DbException e, long rowId, String values) {
         StringBuilder buff = new StringBuilder();
         if (sqlStatement != null) {
             buff.append(sqlStatement);
@@ -492,4 +492,17 @@ public abstract class Prepared {
      */
     public void collectDependencies(HashSet<DbObject> dependencies) {}
 
+    protected final Database getDatabase() {
+        return session.getDatabase();
+    }
+
+    /**
+     * Returns is this command can be repeated again on locking failure.
+     *
+     * @return is this command can be repeated again on locking failure
+     */
+    public boolean isRetryable() {
+        return true;
+    }
+
 }
