@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -39,7 +39,7 @@ public class AlterDomainRename extends AlterDomain {
                 return 0;
             }
         }
-        session.getDatabase().renameSchemaObject(session, domain, newDomainName);
+        getDatabase().renameSchemaObject(session, domain, newDomainName);
         forAllDependencies(session, domain, null, null, false);
         return 0;
     }
