@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -41,7 +41,7 @@ public class AlterTableRenameConstraint extends AlterTable {
     @Override
     public long update(Table table) {
         Constraint constraint = getSchema().findConstraint(session, constraintName);
-        Database db = session.getDatabase();
+        Database db = getDatabase();
         if (constraint == null || constraint.getConstraintType() == Type.DOMAIN || constraint.getTable() != table) {
             throw DbException.get(ErrorCode.CONSTRAINT_NOT_FOUND_1, constraintName);
         }
