@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -99,9 +99,9 @@ public abstract class CommandWithColumns extends SchemaCommand {
         if (columns != null) {
             for (Column c : columns) {
                 if (c.hasIdentityOptions()) {
-                    int objId = session.getDatabase().allocateObjectId();
+                    int objId = getDatabase().allocateObjectId();
                     c.initializeSequence(session, getSchema(), objId, temporary);
-                    if (!Constants.CLUSTERING_DISABLED.equals(session.getDatabase().getCluster())) {
+                    if (!Constants.CLUSTERING_DISABLED.equals(getDatabase().getCluster())) {
                         throw DbException.getUnsupportedException("CLUSTERING && identity columns");
                     }
                 }
