@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -36,7 +36,7 @@ public class CreateSchema extends DefineCommand {
     @Override
     public long update() {
         session.getUser().checkSchemaAdmin();
-        Database db = session.getDatabase();
+        Database db = getDatabase();
         RightOwner owner = db.findUserOrRole(authorization);
         if (owner == null) {
             throw DbException.get(ErrorCode.USER_OR_ROLE_NOT_FOUND_1, authorization);
