@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
@@ -8,30 +8,28 @@ package org.h2.command.ddl;
 import org.h2.api.ErrorCode;
 import org.h2.command.CommandInterface;
 import org.h2.engine.Database;
-import org.h2.engine.FunctionAlias;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.message.DbException;
+import org.h2.schema.FunctionAlias;
 import org.h2.schema.Schema;
 
 /**
  * This class represents the statement
  * DROP ALIAS
  */
-public class DropFunctionAlias extends SchemaCommand {
+public class DropFunctionAlias extends SchemaOwnerCommand {
 
     private String aliasName;
     private boolean ifExists;
 
-    public DropFunctionAlias(Session session, Schema schema) {
+    public DropFunctionAlias(SessionLocal session, Schema schema) {
         super(session, schema);
     }
 
     @Override
-    public int update() {
-        session.getUser().checkAdmin();
-        session.commit(true);
-        Database db = session.getDatabase();
-        FunctionAlias functionAlias = getSchema().findFunction(aliasName);
+    long update(Schema schema) {
+        Database db = getDatabase();
+        FunctionAlias functionAlias = schema.findFunction(aliasName);
         if (functionAlias == null) {
             if (!ifExists) {
                 throw DbException.get(ErrorCode.FUNCTION_ALIAS_NOT_FOUND_1, aliasName);
