@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -37,7 +37,7 @@ public class DropSynonym extends SchemaOwnerCommand {
                 throw DbException.get(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_1, synonymName);
             }
         } else {
-            session.getDatabase().removeSchemaObject(session, synonym);
+            getDatabase().removeSchemaObject(session, synonym);
         }
         return 0;
     }
