@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
@@ -8,7 +8,8 @@ package org.h2.command.ddl;
 import org.h2.api.ErrorCode;
 import org.h2.command.CommandInterface;
 import org.h2.engine.Database;
-import org.h2.engine.Session;
+import org.h2.engine.RightOwner;
+import org.h2.engine.SessionLocal;
 import org.h2.engine.User;
 import org.h2.message.DbException;
 
@@ -21,7 +22,7 @@ public class DropUser extends DefineCommand {
     private boolean ifExists;
     private String userName;
 
-    public DropUser(Session session) {
+    public DropUser(SessionLocal session) {
         super(session);
     }
 
@@ -34,10 +35,9 @@ public class DropUser extends DefineCommand {
     }
 
     @Override
-    public int update() {
+    public long update() {
         session.getUser().checkAdmin();
-        session.commit(true);
-        Database db = session.getDatabase();
+        Database db = getDatabase();
         User user = db.findUser(userName);
         if (user == null) {
             if (!ifExists) {
@@ -46,8 +46,8 @@ public class DropUser extends DefineCommand {
         } else {
             if (user == session.getUser()) {
                 int adminUserCount = 0;
-                for (User u : db.getAllUsers()) {
-                    if (u.isAdmin()) {
+                for (RightOwner rightOwner : db.getAllUsersAndRoles()) {
+                    if (rightOwner instanceof User && ((User) rightOwner).isAdmin()) {
                         adminUserCount++;
                     }
                 }
