@@ -1,17 +1,17 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
 
 import java.util.ArrayList;
+
 import org.h2.command.CommandInterface;
 import org.h2.command.Prepared;
 import org.h2.engine.Procedure;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.expression.Parameter;
-import org.h2.util.New;
 
 /**
  * This class represents the statement
@@ -22,7 +22,7 @@ public class PrepareProcedure extends DefineCommand {
     private String procedureName;
     private Prepared prepared;
 
-    public PrepareProcedure(Session session) {
+    public PrepareProcedure(SessionLocal session) {
         super(session);
     }
 
@@ -32,7 +32,7 @@ public class PrepareProcedure extends DefineCommand {
     }
 
     @Override
-    public int update() {
+    public long update() {
         Procedure proc = new Procedure(procedureName, prepared);
         prepared.setParameterList(parameters);
         prepared.setPrepareAlways(prepareAlways);
@@ -51,7 +51,7 @@ public class PrepareProcedure extends DefineCommand {
 
     @Override
     public ArrayList<Parameter> getParameters() {
-        return New.arrayList();
+        return new ArrayList<>(0);
     }
 
     @Override
