@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -83,7 +83,7 @@ public class Call extends Prepared {
             for (int i = 0; i < columnCount; i++) {
                 String name = result.getColumnName(i);
                 String alias = result.getAlias(i);
-                Expression e = new ExpressionColumn(session.getDatabase(), new Column(name, result.getColumnType(i)));
+                Expression e = new ExpressionColumn(getDatabase(), new Column(name, result.getColumnType(i)));
                 if (!alias.equals(name)) {
                     e = new Alias(e, alias, false);
                 }
