@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -103,7 +103,7 @@ public final class MergeUsing extends DataChangeStatement {
             if (!nullRow) {
                 Row targetRow = targetTableFilter.get();
                 if (table.isRowLockable()) {
-                    Row lockedRow = table.lockRow(session, targetRow);
+                    Row lockedRow = table.lockRow(session, targetRow, -1);
                     if (lockedRow == null) {
                         if (previousSource != source) {
                             missedSource = source;
@@ -179,8 +179,8 @@ public final class MergeUsing extends DataChangeStatement {
     public String getPlanSQL(int sqlFlags) {
         StringBuilder builder = new StringBuilder("MERGE INTO ");
         targetTableFilter.getPlanSQL(builder, false, sqlFlags);
-        builder.append('\n').append("USING ");
-        sourceTableFilter.getPlanSQL(builder, false, sqlFlags);
+        sourceTableFilter.getPlanSQL(builder.append('\n').append("USING "), false, sqlFlags);
+        onCondition.getSQL(builder.append('\n').append("ON "), sqlFlags);
         for (When w : when) {
             w.getSQL(builder.append('\n'), sqlFlags);
         }
