@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -111,7 +111,7 @@ abstract class ScriptBase extends Prepared {
     }
 
     private void initStore() {
-        Database db = session.getDatabase();
+        Database db = getDatabase();
         byte[] key = null;
         if (cipher != null && password != null) {
             char[] pass = password.optimize(session).
