@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -147,7 +147,7 @@ public class Select extends Query {
     boolean isGroupQuery;
     private boolean isGroupSortedQuery;
     private boolean isWindowQuery;
-    private boolean isForUpdate;
+    private ForUpdate forUpdate;
     private double cost;
     private boolean isQuickAggregateQuery, isDistinctQuery;
     private boolean sortUsingIndex;
@@ -429,7 +429,7 @@ public class Select extends Query {
                     Table table = tableFilter.getTable();
                     // Views, function tables, links, etc. do not support locks
                     if (table.isRowLockable()) {
-                        Row lockedRow = table.lockRow(session, row);
+                        Row lockedRow = table.lockRow(session, row, forUpdate.getTimeoutMillis());
                         if (lockedRow == null) {
                             return false;
                         }
@@ -516,7 +516,7 @@ public class Select extends Query {
         setCurrentRowNumber(0);
         while (topTableFilter.next()) {
             setCurrentRowNumber(rowNumber + 1);
-            if (isForUpdate ? isConditionMetForUpdate() : isConditionMet()) {
+            if (forUpdate != null ? isConditionMetForUpdate() : isConditionMet()) {
                 rowNumber++;
                 groupData.nextSource();
                 updateAgg(columnCount, stage);
@@ -626,7 +626,7 @@ public class Select extends Query {
         ArrayList<Index> list = topTableFilter.getTable().getIndexes();
         if (list != null) {
             int[] sortTypes = sort.getSortTypesWithNullOrdering();
-            DefaultNullOrdering defaultNullOrdering = session.getDatabase().getDefaultNullOrdering();
+            DefaultNullOrdering defaultNullOrdering = getDatabase().getDefaultNullOrdering();
             loop: for (Index index : list) {
                 if (index.getCreateSQL() == null) {
                     // can't use the scan index
@@ -716,7 +716,7 @@ public class Select extends Query {
                 limitRows = Long.MAX_VALUE;
             }
         }
-        LazyResultQueryFlat lazyResult = new LazyResultQueryFlat(expressionArray, columnCount, isForUpdate);
+        LazyResultQueryFlat lazyResult = new LazyResultQueryFlat(expressionArray, columnCount, forUpdate != null);
         skipOffset(lazyResult, offset, quickOffset);
         if (result == null) {
             return lazyResult;
@@ -770,12 +770,12 @@ public class Select extends Query {
         long fetch = offsetFetch.fetch;
         boolean fetchPercent = offsetFetch.fetchPercent;
         boolean lazy = session.isLazyQueryExecution() &&
-                target == null && !isForUpdate && !isQuickAggregateQuery &&
+                target == null && forUpdate == null && !isQuickAggregateQuery &&
                 fetch != 0 && !fetchPercent && !withTies && offset == 0 && isReadOnly();
         int columnCount = expressions.size();
         LocalResult result = null;
         if (!lazy && (target == null ||
-                !session.getDatabase().getSettings().optimizeInsertFromSelect)) {
+                !getDatabase().getSettings().optimizeInsertFromSelect)) {
             result = createLocalResult(result);
         }
         // Do not add rows before OFFSET to result if possible
@@ -788,9 +788,9 @@ public class Select extends Query {
             }
         }
         if (distinct) {
+            result = createLocalResult(result);
             if (!isDistinctQuery) {
                 quickOffset = false;
-                result = createLocalResult(result);
                 result.setDistinct();
             }
         } else if (distinctExpressions != null) {
@@ -894,7 +894,7 @@ public class Select extends Query {
                     i = expandColumnList(filter, i, false, exceptTableColumns);
                 }
             } else {
-                Database db = session.getDatabase();
+                Database db = getDatabase();
                 String schemaName = w.getSchemaName();
                 TableFilter filter = null;
                 for (TableFilter f : filters) {
@@ -935,7 +935,7 @@ public class Select extends Query {
                     Column left = entry.getKey(), right = entry.getValue();
                     if (!filter.isCommonJoinColumnToExclude(right)
                             && (except == null || except.remove(left) == null && except.remove(right) == null)) {
-                        Database database = session.getDatabase();
+                        Database database = getDatabase();
                         Expression e;
                         if (left == right
                                 || DataType.hasTotalOrdering(left.getType().getValueType())
@@ -967,7 +967,7 @@ public class Select extends Query {
     private int addExpandedColumn(TableFilter filter, int index, HashMap<Column, ExpressionColumn> except,
             String schema, String alias, Column c) {
         if ((except == null || except.remove(c) == null) && c.getVisible()) {
-            ExpressionColumn ec = new ExpressionColumn(session.getDatabase(), schema, alias, filter.getColumnName(c));
+            ExpressionColumn ec = new ExpressionColumn(getDatabase(), schema, alias, filter.getColumnName(c));
             expressions.add(index++, ec);
         }
         return index;
@@ -1030,7 +1030,7 @@ public class Select extends Query {
             throw DbException.get(ErrorCode.WITH_TIES_WITHOUT_ORDER_BY);
         }
 
-        Database db = session.getDatabase();
+        Database db = getDatabase();
 
         // first the select list (visible columns),
         // then 'ORDER BY' expressions,
@@ -1200,7 +1200,7 @@ public class Select extends Query {
             }
         }
         cost = preparePlan(session.isParsingCreateView());
-        if (distinct && session.getDatabase().getSettings().optimizeDistinct &&
+        if (distinct && getDatabase().getSettings().optimizeDistinct &&
                 !isGroupQuery && filters.size() == 1 &&
                 expressions.size() == 1 && condition == null) {
             Expression expr = expressions.get(0);
@@ -1254,7 +1254,7 @@ public class Select extends Query {
                     }
                 }
             }
-            if (sortUsingIndex && isForUpdate && !topTableFilter.getIndex().isRowIdIndex()) {
+            if (sortUsingIndex && forUpdate != null && !topTableFilter.getIndex().isRowIdIndex()) {
                 sortUsingIndex = false;
             }
         }
@@ -1458,8 +1458,8 @@ public class Select extends Query {
             getFilterSQL(builder, "\nQUALIFY ", exprList, qualify, qualifyIndex, sqlFlags);
         }
         appendEndOfQueryToSQL(builder, sqlFlags, exprList);
-        if (isForUpdate) {
-            builder.append("\nFOR UPDATE");
+        if (forUpdate != null) {
+            forUpdate.getSQL(builder, sqlFlags);
         }
         if ((sqlFlags & ADD_PLAN_INFORMATION) != 0) {
             if (isQuickAggregateQuery) {
@@ -1540,11 +1540,16 @@ public class Select extends Query {
     }
 
     @Override
-    public void setForUpdate(boolean b) {
-        if (b && (isAnyDistinct() || isGroupQuery)) {
+    public ForUpdate getForUpdate() {
+        return forUpdate;
+    }
+
+    @Override
+    public void setForUpdate(ForUpdate b) {
+        if (b != null && (isAnyDistinct() || isGroupQuery)) {
             throw DbException.get(ErrorCode.FOR_UPDATE_IS_NOT_ALLOWED_IN_DISTINCT_OR_GROUPED_SELECT);
         }
-        this.isForUpdate = b;
+        this.forUpdate = b;
     }
 
     @Override
@@ -1673,7 +1678,7 @@ public class Select extends Query {
     public boolean isEverything(ExpressionVisitor visitor) {
         switch (visitor.getType()) {
         case ExpressionVisitor.DETERMINISTIC: {
-            if (isForUpdate) {
+            if (forUpdate != null) {
                 return false;
             }
             for (TableFilter f : filters) {
@@ -1691,7 +1696,7 @@ public class Select extends Query {
             break;
         }
         case ExpressionVisitor.EVALUATABLE: {
-            if (!session.getDatabase().getSettings().optimizeEvaluatableSubqueries) {
+            if (!getDatabase().getSettings().optimizeEvaluatableSubqueries) {
                 return false;
             }
             break;
@@ -1712,6 +1717,12 @@ public class Select extends Query {
                 return false;
             }
         }
+        for (TableFilter f : filters) {
+            Expression c = f.getJoinCondition();
+            if (c != null && !c.isEverything(v2)) {
+                return false;
+            }
+        }
         if (condition != null && !condition.isEverything(v2)) {
             return false;
         }
@@ -1727,7 +1738,7 @@ public class Select extends Query {
 
     @Override
     public boolean isCacheable() {
-        return !isForUpdate;
+        return forUpdate == null;
     }
 
     @Override
