@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -227,7 +227,7 @@ public final class Database implements DataHandler, CastDataProvider {
         this.filePasswordHash = ci.getFilePasswordHash();
         this.databaseName = databaseName;
         this.databaseShortName = parseDatabaseShortName();
-        this.maxLengthInplaceLob = Constants.DEFAULT_MAX_LENGTH_INPLACE_LOB;
+        this.maxLengthInplaceLob = persistent ? Constants.DEFAULT_MAX_LENGTH_INPLACE_LOB : Integer.MAX_VALUE - 8;
         this.cipher = cipher;
         this.autoServerMode = ci.getProperty("AUTO_SERVER", false);
         this.autoServerPort = ci.getProperty("AUTO_SERVER_PORT", 0);
@@ -245,7 +245,7 @@ public final class Database implements DataHandler, CastDataProvider {
         this.databaseURL = ci.getURL();
         String s = ci.removeProperty("DATABASE_EVENT_LISTENER", null);
         if (s != null) {
-            setEventListenerClass(StringUtils.trim(s, true, true, "'"));
+            setEventListenerClass(StringUtils.trim(s, true, true, '\''));
         }
         s = ci.removeProperty("MODE", null);
         if (s != null) {
@@ -264,7 +264,7 @@ public final class Database implements DataHandler, CastDataProvider {
         }
         s = ci.getProperty("JAVA_OBJECT_SERIALIZER", null);
         if (s != null) {
-            s = StringUtils.trim(s, true, true, "'");
+            s = StringUtils.trim(s, true, true, '\'');
             javaObjectSerializerName = s;
         }
         this.allowBuiltinAliasOverride = ci.getProperty("BUILTIN_ALIAS_OVERRIDE", false);
@@ -1249,8 +1249,10 @@ public final class Database implements DataHandler, CastDataProvider {
      */
     private synchronized void closeOpenFilesAndUnlock() {
         try {
-            lobStorage.close();
-            if (!store.getMvStore().isClosed()) {
+            if (lobStorage != null) {
+                lobStorage.close();
+            }
+            if (store != null && !store.getMvStore().isClosed()) {
                 if (compactMode == CommandInterface.SHUTDOWN_IMMEDIATELY) {
                     store.closeImmediately();
                 } else {
@@ -1260,13 +1262,13 @@ public final class Database implements DataHandler, CastDataProvider {
                             dbSettings.defragAlways ? -1 : dbSettings.maxCompactTime;
                     store.close(allowedCompactionTime);
                 }
-            }
-            if (persistent) {
-                // Don't delete temp files if everything is already closed
-                // (maybe in checkPowerOff), the database could be open now
-                // (even from within another process).
-                if (lock != null || fileLockMethod == FileLockMethod.NO || fileLockMethod == FileLockMethod.FS) {
-                    deleteOldTempFiles();
+                if (persistent) {
+                    // Don't delete temp files if everything is already closed
+                    // (maybe in checkPowerOff), the database could be open now
+                    // (even from within another process).
+                    if (lock != null || fileLockMethod == FileLockMethod.NO || fileLockMethod == FileLockMethod.FS) {
+                        deleteOldTempFiles();
+                    }
                 }
             }
         } finally {
@@ -1775,12 +1777,10 @@ public final class Database implements DataHandler, CastDataProvider {
      * that thread, throw it now.
      */
     void throwLastBackgroundException() {
-        if (!store.getMvStore().isBackgroundThread()) {
-            DbException b = backgroundException.getAndSet(null);
-            if (b != null) {
-                // wrap the exception, so we see it was thrown here
-                throw DbException.get(b.getErrorCode(), b, b.getMessage());
-            }
+        DbException b = backgroundException.getAndSet(null);
+        if (b != null) {
+            // wrap the exception, so we see it was thrown here
+            throw DbException.get(b.getErrorCode(), b, b.getMessage());
         }
     }
 
