@@ -1,12 +1,11 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.engine;
 
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -20,15 +19,9 @@ import java.util.Map.Entry;
 public class QueryStatisticsData {
 
     private static final Comparator<QueryEntry> QUERY_ENTRY_COMPARATOR =
-            new Comparator<QueryEntry>() {
-        @Override
-        public int compare(QueryEntry o1, QueryEntry o2) {
-            return (int) Math.signum(o1.lastUpdateTime - o2.lastUpdateTime);
-        }
-    };
+            Comparator.comparingLong(q -> q.lastUpdateTime);
 
-    private final HashMap<String, QueryEntry> map =
-            new HashMap<String, QueryEntry>();
+    private final HashMap<String, QueryEntry> map = new HashMap<>();
 
     private int maxQueryEntries;
 
@@ -43,10 +36,9 @@ public class QueryStatisticsData {
     public synchronized List<QueryEntry> getQueries() {
         // return a copy of the map so we don't have to
         // worry about external synchronization
-        ArrayList<QueryEntry> list = new ArrayList<QueryEntry>();
-        list.addAll(map.values());
+        ArrayList<QueryEntry> list = new ArrayList<>(map.values());
         // only return the newest 100 entries
-        Collections.sort(list, QUERY_ENTRY_COMPARATOR);
+        list.sort(QUERY_ENTRY_COMPARATOR);
         return list.subList(0, Math.min(list.size(), maxQueryEntries));
     }
 
@@ -58,8 +50,7 @@ public class QueryStatisticsData {
      *            to execute
      * @param rowCount the query or update row count
      */
-    public synchronized void update(String sqlStatement, long executionTimeNanos,
-            int rowCount) {
+    public synchronized void update(String sqlStatement, long executionTimeNanos, long rowCount) {
         QueryEntry entry = map.get(sqlStatement);
         if (entry == null) {
             entry = new QueryEntry(sqlStatement);
@@ -71,12 +62,11 @@ public class QueryStatisticsData {
         // Test against 1.5 x max-size so we don't do this too often
         if (map.size() > maxQueryEntries * 1.5f) {
             // Sort the entries by age
-            ArrayList<QueryEntry> list = new ArrayList<QueryEntry>();
-            list.addAll(map.values());
-            Collections.sort(list, QUERY_ENTRY_COMPARATOR);
+            ArrayList<QueryEntry> list = new ArrayList<>(map.values());
+            list.sort(QUERY_ENTRY_COMPARATOR);
             // Create a set of the oldest 1/3 of the entries
             HashSet<QueryEntry> oldestSet =
-                    new HashSet<QueryEntry>(list.subList(0, list.size() / 3));
+                    new HashSet<>(list.subList(0, list.size() / 3));
             // Loop over the map using the set and remove
             // the oldest 1/3 of the entries.
             for (Iterator<Entry<String, QueryEntry>> it =
@@ -128,12 +118,12 @@ public class QueryStatisticsData {
         /**
          * The minimum number of rows.
          */
-        public int rowCountMin;
+        public long rowCountMin;
 
         /**
          * The maximum number of rows.
          */
-        public int rowCountMax;
+        public long rowCountMax;
 
         /**
          * The total number of rows.
@@ -151,8 +141,8 @@ public class QueryStatisticsData {
         public double rowCountMean;
 
         // Using Welford's method, see also
-        // http://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
-        // http://www.johndcook.com/standard_deviation.html
+        // https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
+        // https://www.johndcook.com/blog/standard_deviation/
 
         private double executionTimeM2Nanos;
         private double rowCountM2;
@@ -167,7 +157,7 @@ public class QueryStatisticsData {
          * @param timeNanos the execution time in nanos
          * @param rows the number of rows
          */
-        void update(long timeNanos, int rows) {
+        void update(long timeNanos, long rows) {
             count++;
             executionTimeMinNanos = Math.min(timeNanos, executionTimeMinNanos);
             executionTimeMaxNanos = Math.max(timeNanos, executionTimeMaxNanos);
