@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -408,7 +408,7 @@ public final class SessionRemote extends Session implements DataHandler {
         if (autoReconnect) {
             String className = ci.getProperty("DATABASE_EVENT_LISTENER");
             if (className != null) {
-                className = StringUtils.trim(className, true, true, "'");
+                className = StringUtils.trim(className, true, true, '\'');
                 try {
                     eventListener = (DatabaseEventListener) JdbcUtils
                             .loadUserClass(className).getDeclaredConstructor().newInstance();
