@@ -1,11 +1,12 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.expression;
 
 import org.h2.message.DbException;
+import org.h2.value.TypeInfo;
 import org.h2.value.Value;
 
 /**
@@ -43,26 +44,12 @@ public interface ParameterInterface {
     boolean isValueSet();
 
     /**
-     * Get the expected data type of the parameter if no value is set, or the
+     * Returns the expected data type if no value is set, or the
      * data type of the value if one is set.
      *
      * @return the data type
      */
-    int getType();
-
-    /**
-     * Get the expected precision of this parameter.
-     *
-     * @return the expected precision
-     */
-    long getPrecision();
-
-    /**
-     * Get the expected scale of this parameter.
-     *
-     * @return the expected scale
-     */
-    int getScale();
+    TypeInfo getType();
 
     /**
      * Check if this column is nullable.
