@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.jdbc;
@@ -9,14 +9,12 @@ import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
-import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.PipedInputStream;
-import java.io.PipedOutputStream;
 import java.sql.Blob;
 import java.sql.SQLException;
-import org.h2.api.ErrorCode;
+
 import org.h2.engine.Constants;
 import org.h2.message.DbException;
 import org.h2.message.TraceObject;
@@ -27,18 +25,17 @@ import org.h2.value.Value;
 /**
  * Represents a BLOB value.
  */
-public class JdbcBlob extends TraceObject implements Blob {
-
-    Value value;
-    private final JdbcConnection conn;
+public final class JdbcBlob extends JdbcLob implements Blob {
 
     /**
      * INTERNAL
+     * @param conn it belongs to
+     * @param value of
+     * @param state of the LOB
+     * @param id of the trace object
      */
-    public JdbcBlob(JdbcConnection conn, Value value, int id) {
-        setTrace(conn.getSession().getTrace(), TraceObject.BLOB, id);
-        this.conn = conn;
-        this.value = value;
+    public JdbcBlob(JdbcConnection conn, Value value, State state, int id) {
+        super(conn, value, state, TraceObject.BLOB, id);
     }
 
     /**
@@ -50,9 +47,9 @@ public class JdbcBlob extends TraceObject implements Blob {
     public long length() throws SQLException {
         try {
             debugCodeCall("length");
-            checkClosed();
-            if (value.getType() == Value.BLOB) {
-                long precision = value.getPrecision();
+            checkReadable();
+            if (value.getValueType() == Value.BLOB) {
+                long precision = value.getType().getPrecision();
                 if (precision > 0) {
                     return precision;
                 }
@@ -84,9 +81,9 @@ public class JdbcBlob extends TraceObject implements Blob {
     public byte[] getBytes(long pos, int length) throws SQLException {
         try {
             if (isDebugEnabled()) {
-                debugCode("getBytes("+pos+", "+length+");");
+                debugCode("getBytes(" + pos + ", " + length + ')');
             }
-            checkClosed();
+            checkReadable();
             ByteArrayOutputStream out = new ByteArrayOutputStream();
             try (InputStream in = value.getInputStream()) {
                 IOUtils.skipFully(in, pos - 1);
@@ -109,15 +106,18 @@ public class JdbcBlob extends TraceObject implements Blob {
      */
     @Override
     public int setBytes(long pos, byte[] bytes) throws SQLException {
+        if (bytes == null) {
+            throw new NullPointerException();
+        }
         try {
             if (isDebugEnabled()) {
-                debugCode("setBytes("+pos+", "+quoteBytes(bytes)+");");
+                debugCode("setBytes(" + pos + ", " + quoteBytes(bytes) + ')');
             }
-            checkClosed();
+            checkEditable();
             if (pos != 1) {
                 throw DbException.getInvalidValueException("pos", pos);
             }
-            value = conn.createBlob(new ByteArrayInputStream(bytes), -1);
+            completeWrite(conn.createBlob(new ByteArrayInputStream(bytes), -1));
             return bytes.length;
         } catch (Exception e) {
             throw logAndConvert(e);
@@ -125,7 +125,7 @@ public class JdbcBlob extends TraceObject implements Blob {
     }
 
     /**
-     * [Not supported] Sets some bytes of the object.
+     * Sets some bytes of the object.
      *
      * @param pos the write position
      * @param bytes the bytes to set
@@ -136,25 +136,29 @@ public class JdbcBlob extends TraceObject implements Blob {
     @Override
     public int setBytes(long pos, byte[] bytes, int offset, int len)
             throws SQLException {
-        throw unsupported("LOB update");
-    }
-
-    /**
-     * Returns the input stream.
-     *
-     * @return the input stream
-     */
-    @Override
-    public InputStream getBinaryStream() throws SQLException {
+        if (bytes == null) {
+            throw new NullPointerException();
+        }
         try {
-            debugCodeCall("getBinaryStream");
-            checkClosed();
-            return value.getInputStream();
+            if (isDebugEnabled()) {
+                debugCode("setBytes(" + pos + ", " + quoteBytes(bytes) + ", " + offset + ", " + len + ')');
+            }
+            checkEditable();
+            if (pos != 1) {
+                throw DbException.getInvalidValueException("pos", pos);
+            }
+            completeWrite(conn.createBlob(new ByteArrayInputStream(bytes, offset, len), -1));
+            return (int) value.getType().getPrecision();
         } catch (Exception e) {
             throw logAndConvert(e);
         }
     }
 
+    @Override
+    public InputStream getBinaryStream() throws SQLException {
+        return super.getBinaryStream();
+    }
+
     /**
      * Get a writer to update the Blob. This is only supported for new, empty
      * Blob objects that were created with Connection.createBlob(). The Blob is
@@ -169,35 +173,22 @@ public class JdbcBlob extends TraceObject implements Blob {
     public OutputStream setBinaryStream(long pos) throws SQLException {
         try {
             if (isDebugEnabled()) {
-                debugCode("setBinaryStream("+pos+");");
+                debugCodeCall("setBinaryStream", pos);
             }
-            checkClosed();
+            checkEditable();
             if (pos != 1) {
                 throw DbException.getInvalidValueException("pos", pos);
             }
-            if (value.getPrecision() != 0) {
-                throw DbException.getInvalidValueException("length", value.getPrecision());
-            }
-            final JdbcConnection c = conn;
             final PipedInputStream in = new PipedInputStream();
             final Task task = new Task() {
                 @Override
                 public void call() {
-                    value = c.createBlob(in, -1);
-                }
-            };
-            PipedOutputStream out = new PipedOutputStream(in) {
-                @Override
-                public void close() throws IOException {
-                    super.close();
-                    try {
-                        task.get();
-                    } catch (Exception e) {
-                        throw DbException.convertToIOException(e);
-                    }
+                    completeWrite(conn.createBlob(in, -1));
                 }
             };
+            LobPipedOutputStream out = new LobPipedOutputStream(in, task);
             task.execute();
+            state = State.SET_CALLED;
             return new BufferedOutputStream(out);
         } catch (Exception e) {
             throw logAndConvert(e);
@@ -214,11 +205,11 @@ public class JdbcBlob extends TraceObject implements Blob {
     @Override
     public long position(byte[] pattern, long start) throws SQLException {
         if (isDebugEnabled()) {
-            debugCode("position("+quoteBytes(pattern)+", "+start+");");
+            debugCode("position(" + quoteBytes(pattern) + ", " + start + ')');
         }
         if (Constants.BLOB_SEARCH) {
             try {
-                checkClosed();
+                checkReadable();
                 if (pattern == null) {
                     return -1;
                 }
@@ -269,11 +260,11 @@ public class JdbcBlob extends TraceObject implements Blob {
     @Override
     public long position(Blob blobPattern, long start) throws SQLException {
         if (isDebugEnabled()) {
-            debugCode("position(blobPattern, "+start+");");
+            debugCode("position(blobPattern, " + start + ')');
         }
         if (Constants.BLOB_SEARCH) {
             try {
-                checkClosed();
+                checkReadable();
                 if (blobPattern == null) {
                     return -1;
                 }
@@ -295,16 +286,7 @@ public class JdbcBlob extends TraceObject implements Blob {
     }
 
     /**
-     * Release all resources of this object.
-     */
-    @Override
-    public void free() {
-        debugCodeCall("free");
-        value = null;
-    }
-
-    /**
-     * [Not supported] Returns the input stream, starting from an offset.
+     * Returns the input stream, starting from an offset.
      *
      * @param pos where to start reading
      * @param length the number of bytes that will be read
@@ -312,23 +294,23 @@ public class JdbcBlob extends TraceObject implements Blob {
      */
     @Override
     public InputStream getBinaryStream(long pos, long length) throws SQLException {
-        throw unsupported("LOB update");
-    }
-
-    private void checkClosed() {
-        conn.checkClosed();
-        if (value == null) {
-            throw DbException.get(ErrorCode.OBJECT_CLOSED);
+        try {
+            if (isDebugEnabled()) {
+                debugCode("getBinaryStream(" + pos + ", " + length + ')');
+            }
+            checkReadable();
+            if (state == State.NEW) {
+                if (pos != 1) {
+                    throw DbException.getInvalidValueException("pos", pos);
+                }
+                if (length != 0) {
+                    throw DbException.getInvalidValueException("length", pos);
+                }
+            }
+            return value.getInputStream(pos, length);
+        } catch (Exception e) {
+            throw logAndConvert(e);
         }
     }
 
-    /**
-     * INTERNAL
-     */
-    @Override
-    public String toString() {
-        return getTraceObjectName() + ": " +
-                (value == null ? "null" : value.getTraceSQL());
-    }
-
 }
