@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -94,7 +94,7 @@ public final class Trace {
     /**
      * Module names by their ids as array indexes.
      */
-    public static final String[] MODULE_NAMES = {
+    static final String[] MODULE_NAMES = {
         "command",
         "constraint",
         "database",
