@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: Daniel Gredler
  */
@@ -617,7 +617,24 @@ final class ToDateTokenizer {
 
         private static final List<FormatTokenEnum> INLINE_LIST = Collections.singletonList(INLINE);
 
-        private static List<FormatTokenEnum>[] TOKENS;
+        private static final List<FormatTokenEnum>[] TOKENS;
+
+        static {
+            @SuppressWarnings("unchecked")
+            List<FormatTokenEnum>[] tokens = new List[25];
+            for (FormatTokenEnum token : FormatTokenEnum.values()) {
+                String name = token.name();
+                if (name.indexOf('_') >= 0) {
+                    for (String tokenLets : name.split("_")) {
+                        putToCache(tokens, token, tokenLets);
+                    }
+                } else {
+                    putToCache(tokens, token, name);
+                }
+            }
+            TOKENS = tokens;
+        }
+
         private final ToDateParslet toDateParslet;
         private final Pattern patternToUse;
 
@@ -655,11 +672,7 @@ final class ToDateTokenizer {
             if (formatStr != null && !formatStr.isEmpty()) {
                 char key = Character.toUpperCase(formatStr.charAt(0));
                 if (key >= 'A' && key <= 'Y') {
-                    List<FormatTokenEnum>[] tokens = TOKENS;
-                    if (tokens == null) {
-                        tokens = initTokens();
-                    }
-                    return tokens[key - 'A'];
+                    return TOKENS[key - 'A'];
                 } else if (key == '"') {
                     return INLINE_LIST;
                 }
@@ -667,22 +680,6 @@ final class ToDateTokenizer {
             return null;
         }
 
-        @SuppressWarnings("unchecked")
-        private static List<FormatTokenEnum>[] initTokens() {
-            List<FormatTokenEnum>[] tokens = new List[25];
-            for (FormatTokenEnum token : FormatTokenEnum.values()) {
-                String name = token.name();
-                if (name.indexOf('_') >= 0) {
-                    for (String tokenLets : name.split("_")) {
-                        putToCache(tokens, token, tokenLets);
-                    }
-                } else {
-                    putToCache(tokens, token, name);
-                }
-            }
-            return TOKENS = tokens;
-        }
-
         private static void putToCache(List<FormatTokenEnum>[] cache, FormatTokenEnum token, String name) {
             int idx = Character.toUpperCase(name.charAt(0)) - 'A';
             List<FormatTokenEnum> l = cache[idx];
