@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.mvstore;
@@ -10,21 +10,20 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
-import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 import java.text.MessageFormat;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.ConcurrentModificationException;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Map;
 
 import org.h2.engine.Constants;
-import org.h2.util.New;
+import org.h2.jdbc.JdbcException;
+import org.h2.util.StringUtils;
 
 /**
  * Utility methods
  */
-public class DataUtils {
+public final class DataUtils {
 
     /**
      * An error occurred while reading from the file.
@@ -99,6 +98,22 @@ public class DataUtils {
      */
     public static final int ERROR_TRANSACTION_ILLEGAL_STATE = 103;
 
+    /**
+     * The transaction contains too many changes.
+     */
+    public static final int ERROR_TRANSACTION_TOO_BIG = 104;
+
+    /**
+     * Deadlock discovered and one of transactions involved chosen as victim and rolled back.
+     */
+    public static final int ERROR_TRANSACTIONS_DEADLOCK = 105;
+
+    /**
+     * The transaction store can not be initialized because data type
+     * is not found in type registry.
+     */
+    public static final int ERROR_UNKNOWN_DATA_TYPE = 106;
+
     /**
      * The type for leaf page.
      */
@@ -119,16 +134,6 @@ public class DataUtils {
      */
     public static final int PAGE_COMPRESSED_HIGH = 2 + 4;
 
-    /**
-     * The maximum length of a variable size int.
-     */
-    public static final int MAX_VAR_INT_LEN = 5;
-
-    /**
-     * The maximum length of a variable size long.
-     */
-    public static final int MAX_VAR_LONG_LEN = 10;
-
     /**
      * The maximum integer that needs less space when using variable size
      * encoding (only 3 bytes instead of 4).
@@ -141,40 +146,38 @@ public class DataUtils {
      */
     public static final long COMPRESSED_VAR_LONG_MAX = 0x1ffffffffffffL;
 
-    /**
-     * The estimated number of bytes used per page object.
-     */
-    public static final int PAGE_MEMORY = 128;
-
-    /**
-     * The estimated number of bytes used per child entry.
-     */
-    public static final int PAGE_MEMORY_CHILD = 16;
-
     /**
      * The marker size of a very large page.
      */
     public static final int PAGE_LARGE = 2 * 1024 * 1024;
 
-    /**
-     * The UTF-8 character encoding format.
-     */
-    public static final Charset UTF8 = Charset.forName("UTF-8");
+    // The following are key prefixes used in layout map
 
     /**
-     * The ISO Latin character encoding format.
+     * The prefix for chunks ("chunk."). This, plus the chunk id (hex encoded)
+     * is the key, and the serialized chunk metadata is the value.
      */
-    public static final Charset LATIN = Charset.forName("ISO-8859-1");
+    public static final String META_CHUNK = "chunk.";
 
     /**
-     * An 0-size byte array.
+     * The prefix for root positions of maps ("root."). This, plus the map id
+     * (hex encoded) is the key, and the position (hex encoded) is the value.
      */
-    private static final byte[] EMPTY_BYTES = {};
+    public static final String META_ROOT = "root.";
+
+    // The following are key prefixes used in meta map
 
     /**
-     * The maximum byte to grow a buffer at a time.
+     * The prefix for names ("name."). This, plus the name of the map, is the
+     * key, and the map id (hex encoded) is the value.
      */
-    private static final int MAX_GROW = 16 * 1024 * 1024;
+    public static final String META_NAME = "name.";
+
+    /**
+     * The prefix for maps ("map."). This, plus the map id (hex encoded) is the
+     * key, and the serialized in the map metadata is the value.
+     */
+    public static final String META_MAP = "map.";
 
     /**
      * Get the length of the variable size int.
@@ -274,10 +277,11 @@ public class DataUtils {
      *
      * @param out the output stream
      * @param x the value
+     * @throws IOException if some data could not be written
      */
     public static void writeVarInt(OutputStream out, int x) throws IOException {
         while ((x & ~0x7f) != 0) {
-            out.write((byte) (0x80 | (x & 0x7f)));
+            out.write((byte) (x | 0x80));
             x >>>= 7;
         }
         out.write((byte) x);
@@ -291,7 +295,7 @@ public class DataUtils {
      */
     public static void writeVarInt(ByteBuffer buff, int x) {
         while ((x & ~0x7f) != 0) {
-            buff.put((byte) (0x80 | (x & 0x7f)));
+            buff.put((byte) (x | 0x80));
             x >>>= 7;
         }
         buff.put((byte) x);
@@ -300,28 +304,35 @@ public class DataUtils {
     /**
      * Write characters from a string (without the length).
      *
-     * @param buff the target buffer
+     * @param buff the target buffer (must be large enough)
      * @param s the string
      * @param len the number of characters
-     * @return the byte buffer
      */
-    public static ByteBuffer writeStringData(ByteBuffer buff,
+    public static void writeStringData(ByteBuffer buff,
             String s, int len) {
-        buff = DataUtils.ensureCapacity(buff, 3 * len);
         for (int i = 0; i < len; i++) {
             int c = s.charAt(i);
             if (c < 0x80) {
                 buff.put((byte) c);
             } else if (c >= 0x800) {
                 buff.put((byte) (0xe0 | (c >> 12)));
-                buff.put((byte) (((c >> 6) & 0x3f)));
+                buff.put((byte) ((c >> 6) & 0x3f));
                 buff.put((byte) (c & 0x3f));
             } else {
                 buff.put((byte) (0xc0 | (c >> 6)));
                 buff.put((byte) (c & 0x3f));
             }
         }
-        return buff;
+    }
+
+    /**
+     * Read a string.
+     *
+     * @param buff the source buffer
+     * @return the value
+     */
+    public static String readString(ByteBuffer buff) {
+        return readString(buff, readVarInt(buff));
     }
 
     /**
@@ -355,7 +366,7 @@ public class DataUtils {
      */
     public static void writeVarLong(ByteBuffer buff, long x) {
         while ((x & ~0x7f) != 0) {
-            buff.put((byte) (0x80 | (x & 0x7f)));
+            buff.put((byte) (x | 0x80));
             x >>>= 7;
         }
         buff.put((byte) x);
@@ -366,11 +377,12 @@ public class DataUtils {
      *
      * @param out the output stream
      * @param x the value
+     * @throws IOException if some data could not be written
      */
     public static void writeVarLong(OutputStream out, long x)
             throws IOException {
         while ((x & ~0x7f) != 0) {
-            out.write((byte) (0x80 | (x & 0x7f)));
+            out.write((byte) (x | 0x80));
             x >>>= 7;
         }
         out.write((byte) x);
@@ -421,7 +433,7 @@ public class DataUtils {
      * @param file the file channel
      * @param pos the absolute position within the file
      * @param dst the byte buffer
-     * @throws IllegalStateException if some data could not be read
+     * @throws MVStoreException if some data could not be read
      */
     public static void readFully(FileChannel file, long pos, ByteBuffer dst) {
         try {
@@ -440,11 +452,11 @@ public class DataUtils {
             } catch (IOException e2) {
                 size = -1;
             }
-            throw newIllegalStateException(
+            throw newMVStoreException(
                     ERROR_READING_FAILED,
-                    "Reading from {0} failed; file length {1} " +
-                    "read length {2} at {3}",
-                    file, size, dst.remaining(), pos, e);
+                    "Reading from file {0} failed at {1} (length {2}), " +
+                    "read {3}, remaining {4}",
+                    file, pos, size, dst.position(), dst.remaining(), e);
         }
     }
 
@@ -463,7 +475,7 @@ public class DataUtils {
                 off += len;
             } while (src.remaining() > 0);
         } catch (IOException e) {
-            throw newIllegalStateException(
+            throw newMVStoreException(
                     ERROR_WRITING_FAILED,
                     "Writing to {0} failed; length {1} at {2}",
                     file, src.remaining(), pos, e);
@@ -477,6 +489,7 @@ public class DataUtils {
      * @return the length code
      */
     public static int encodeLength(int len) {
+        assert len >= 0;
         if (len <= 32) {
             return 0;
         }
@@ -514,14 +527,34 @@ public class DataUtils {
     }
 
     /**
-     * Get the maximum length for the given code.
-     * For the code 31, PAGE_LARGE is returned.
+     * Get the map id from the chunk's table of content element.
+     *
+     * @param tocElement packed table of content element
+     * @return the map id
+     */
+    public static int getPageMapId(long tocElement) {
+        return (int) (tocElement >>> 38);
+    }
+
+    /**
+     * Get the maximum length for the given page position.
      *
      * @param pos the position
      * @return the maximum length
      */
     public static int getPageMaxLength(long pos) {
         int code = (int) ((pos >> 1) & 31);
+        return decodePageLength(code);
+    }
+
+    /**
+     * Get the maximum length for the given code.
+     * For the code 31, PAGE_LARGE is returned.
+     *
+     * @param code encoded page length
+     * @return the maximum length
+     */
+    public static int decodePageLength(int code) {
         if (code == 31) {
             return PAGE_LARGE;
         }
@@ -531,11 +564,11 @@ public class DataUtils {
     /**
      * Get the offset from the position.
      *
-     * @param pos the position
+     * @param tocElement packed table of content element
      * @return the offset
      */
-    public static int getPageOffset(long pos) {
-        return (int) (pos >> 6);
+    public static int getPageOffset(long tocElement) {
+        return (int) (tocElement >> 6);
     }
 
     /**
@@ -548,9 +581,39 @@ public class DataUtils {
         return ((int) pos) & 1;
     }
 
+    /**
+     * Determines whether specified file position corresponds to a leaf page
+     * @param pos the position
+     * @return true if it is a leaf, false otherwise
+     */
+    public static boolean isLeafPosition(long pos) {
+        return getPageType(pos) == PAGE_TYPE_LEAF;
+    }
+
+    /**
+     * Find out if page was saved.
+     *
+     * @param pos the position
+     * @return true if page has been saved
+     */
+    public static boolean isPageSaved(long pos) {
+        return (pos & ~1L) != 0;
+    }
+
+    /**
+     * Find out if page was removed.
+     *
+     * @param pos the position
+     * @return true if page has been removed (no longer accessible from the
+     *         current root of the tree)
+     */
+    static boolean isPageRemoved(long pos) {
+        return pos == 1L;
+    }
+
     /**
      * Get the position of this page. The following information is encoded in
-     * the position: the chunk id, the offset, the maximum length, and the type
+     * the position: the chunk id, the page sequential number, the maximum length, and the type
      * (node or leaf).
      *
      * @param chunkId the chunk id
@@ -559,11 +622,46 @@ public class DataUtils {
      * @param type the page type (1 for node, 0 for leaf)
      * @return the position
      */
-    public static long getPagePos(int chunkId, int offset,
-            int length, int type) {
+    public static long composePagePos(int chunkId, int offset, int length, int type) {
+        assert offset >= 0;
+        assert type == DataUtils.PAGE_TYPE_LEAF || type == DataUtils.PAGE_TYPE_NODE;
+
         long pos = (long) chunkId << 38;
         pos |= (long) offset << 6;
-        pos |= encodeLength(length) << 1;
+        pos |= (long) encodeLength(length) << 1;
+        pos |= type;
+        return pos;
+    }
+
+    /**
+     * Convert tocElement into pagePos by replacing mapId with chunkId.
+     *
+     * @param chunkId the chunk id
+     * @param tocElement the element
+     * @return the page position
+     */
+    public static long composePagePos(int chunkId, long tocElement) {
+        return (tocElement & 0x3FFFFFFFFFL) | ((long) chunkId << 38);
+    }
+
+    /**
+     * Create table of content element. The following information is encoded in it:
+     * the map id, the page offset, the maximum length, and the type
+     * (node or leaf).
+     *
+     * @param mapId the chunk id
+     * @param offset the offset
+     * @param length the length
+     * @param type the page type (1 for node, 0 for leaf)
+     * @return the position
+     */
+    public static long composeTocElement(int mapId, int offset, int length, int type) {
+        assert mapId >= 0;
+        assert offset >= 0;
+        assert type == DataUtils.PAGE_TYPE_LEAF || type == DataUtils.PAGE_TYPE_NODE;
+        long pos = (long) mapId << 38;
+        pos |= (long) offset << 6;
+        pos |= (long) encodeLength(length) << 1;
         pos |= type;
         return pos;
     }
@@ -587,16 +685,30 @@ public class DataUtils {
      * @param map the map
      * @return the string builder
      */
-    public static StringBuilder appendMap(StringBuilder buff,
-            HashMap<String, ?> map) {
-        ArrayList<String> list = New.arrayList(map.keySet());
-        Collections.sort(list);
-        for (String k : list) {
-            appendMap(buff, k, map.get(k));
+    public static StringBuilder appendMap(StringBuilder buff, HashMap<String, ?> map) {
+        Object[] keys = map.keySet().toArray();
+        Arrays.sort(keys);
+        for (Object k : keys) {
+            String key = (String) k;
+            Object value = map.get(key);
+            if (value instanceof Long) {
+                appendMap(buff, key, (long) value);
+            } else if (value instanceof Integer) {
+                appendMap(buff, key, (int) value);
+            } else {
+                appendMap(buff, key, value.toString());
+            }
         }
         return buff;
     }
 
+    private static StringBuilder appendMapKey(StringBuilder buff, String key) {
+        if (buff.length() > 0) {
+            buff.append(',');
+        }
+        return buff.append(key).append(':');
+    }
+
     /**
      * Append a key-value pair to the string builder. Keys may not contain a
      * colon. Values that contain a comma or a double quote are enclosed in
@@ -606,25 +718,14 @@ public class DataUtils {
      * @param key the key
      * @param value the value
      */
-    public static void appendMap(StringBuilder buff, String key, Object value) {
-        if (buff.length() > 0) {
-            buff.append(',');
-        }
-        buff.append(key).append(':');
-        String v;
-        if (value instanceof Long) {
-            v = Long.toHexString((Long) value);
-        } else if (value instanceof Integer) {
-            v = Integer.toHexString((Integer) value);
-        } else {
-            v = value.toString();
-        }
-        if (v.indexOf(',') < 0 && v.indexOf('\"') < 0) {
-            buff.append(v);
+    public static void appendMap(StringBuilder buff, String key, String value) {
+        appendMapKey(buff, key);
+        if (value.indexOf(',') < 0 && value.indexOf('\"') < 0) {
+            buff.append(value);
         } else {
             buff.append('\"');
-            for (int i = 0, size = v.length(); i < size; i++) {
-                char c = v.charAt(i);
+            for (int i = 0, size = value.length(); i < size; i++) {
+                char c = value.charAt(i);
                 if (c == '\"') {
                     buff.append('\\');
                 }
@@ -634,72 +735,204 @@ public class DataUtils {
         }
     }
 
+    /**
+     * Append a key-value pair to the string builder. Keys may not contain a
+     * colon.
+     *
+     * @param buff the target buffer
+     * @param key the key
+     * @param value the value
+     */
+    public static void appendMap(StringBuilder buff, String key, long value) {
+        appendMapKey(buff, key).append(Long.toHexString(value));
+    }
+
+    /**
+     * Append a key-value pair to the string builder. Keys may not contain a
+     * colon.
+     *
+     * @param buff the target buffer
+     * @param key the key
+     * @param value the value
+     */
+    public static void appendMap(StringBuilder buff, String key, int value) {
+        appendMapKey(buff, key).append(Integer.toHexString(value));
+    }
+
+    /**
+     * @param buff output buffer, should be empty
+     * @param s parsed string
+     * @param i offset to parse from
+     * @param size stop offset (exclusive)
+     * @return new offset
+     */
+    private static int parseMapValue(StringBuilder buff, String s, int i, int size) {
+        while (i < size) {
+            char c = s.charAt(i++);
+            if (c == ',') {
+                break;
+            } else if (c == '\"') {
+                while (i < size) {
+                    c = s.charAt(i++);
+                    if (c == '\\') {
+                        if (i == size) {
+                            throw newMVStoreException(ERROR_FILE_CORRUPT, "Not a map: {0}", s);
+                        }
+                        c = s.charAt(i++);
+                    } else if (c == '\"') {
+                        break;
+                    }
+                    buff.append(c);
+                }
+            } else {
+                buff.append(c);
+            }
+        }
+        return i;
+    }
+
     /**
      * Parse a key-value pair list.
      *
      * @param s the list
      * @return the map
-     * @throws IllegalStateException if parsing failed
+     * @throws MVStoreException if parsing failed
      */
     public static HashMap<String, String> parseMap(String s) {
-        HashMap<String, String> map = New.hashMap();
+        HashMap<String, String> map = new HashMap<>();
+        StringBuilder buff = new StringBuilder();
         for (int i = 0, size = s.length(); i < size;) {
             int startKey = i;
             i = s.indexOf(':', i);
             if (i < 0) {
-                throw DataUtils.newIllegalStateException(
-                        DataUtils.ERROR_FILE_CORRUPT, "Not a map: {0}", s);
+                throw newMVStoreException(ERROR_FILE_CORRUPT, "Not a map: {0}", s);
             }
             String key = s.substring(startKey, i++);
-            StringBuilder buff = new StringBuilder();
-            while (i < size) {
-                char c = s.charAt(i++);
-                if (c == ',') {
-                    break;
-                } else if (c == '\"') {
-                    while (i < size) {
-                        c = s.charAt(i++);
-                        if (c == '\\') {
-                            if (i == size) {
-                                throw DataUtils.newIllegalStateException(
-                                        DataUtils.ERROR_FILE_CORRUPT,
-                                        "Not a map: {0}", s);
-                            }
-                            c = s.charAt(i++);
-                        } else if (c == '\"') {
-                            break;
-                        }
-                        buff.append(c);
-                    }
-                } else {
-                    buff.append(c);
-                }
-            }
+            i = parseMapValue(buff, s, i, size);
             map.put(key, buff.toString());
+            buff.setLength(0);
         }
         return map;
     }
 
+    /**
+     * Parse a key-value pair list and checks its checksum.
+     *
+     * @param bytes encoded map
+     * @return the map without mapping for {@code "fletcher"}, or {@code null} if checksum is wrong
+     *              or parameter do not represent a properly formatted map serialization
+     */
+    static HashMap<String, String> parseChecksummedMap(byte[] bytes) {
+        int start = 0, end = bytes.length;
+        while (start < end && bytes[start] <= ' ') {
+            start++;
+        }
+        while (start < end && bytes[end - 1] <= ' ') {
+            end--;
+        }
+        String s = new String(bytes, start, end - start, StandardCharsets.ISO_8859_1);
+        HashMap<String, String> map = new HashMap<>();
+        StringBuilder buff = new StringBuilder();
+        for (int i = 0, size = s.length(); i < size;) {
+            int startKey = i;
+            i = s.indexOf(':', i);
+            if (i < 0) {
+                // Corrupted map
+                return null;
+            }
+            if (i - startKey == 8 && s.regionMatches(startKey, "fletcher", 0, 8)) {
+                parseMapValue(buff, s, i + 1, size);
+                int check = (int) Long.parseLong(buff.toString(), 16);
+                if (check == getFletcher32(bytes, start, startKey - 1)) {
+                    return map;
+                }
+                // Corrupted map
+                return null;
+            }
+            String key = s.substring(startKey, i++);
+            i = parseMapValue(buff, s, i, size);
+            map.put(key, buff.toString());
+            buff.setLength(0);
+        }
+        // Corrupted map
+        return null;
+    }
+
+    /**
+     * Parse a name from key-value pair list.
+     *
+     * @param s the list
+     * @return value of name item, or {@code null}
+     * @throws MVStoreException if parsing failed
+     */
+    public static String getMapName(String s) {
+        return getFromMap(s, "name");
+    }
+
+    /**
+     * Parse a specified pair from key-value pair list.
+     *
+     * @param s the list
+     * @param key the name of the key
+     * @return value of the specified item, or {@code null}
+     * @throws MVStoreException if parsing failed
+     */
+    public static String getFromMap(String s, String key) {
+        int keyLength = key.length();
+        for (int i = 0, size = s.length(); i < size;) {
+            int startKey = i;
+            i = s.indexOf(':', i);
+            if (i < 0) {
+                throw newMVStoreException(ERROR_FILE_CORRUPT, "Not a map: {0}", s);
+            }
+            if (i++ - startKey == keyLength && s.regionMatches(startKey, key, 0, keyLength)) {
+                StringBuilder buff = new StringBuilder();
+                parseMapValue(buff, s, i, size);
+                return buff.toString();
+            } else {
+                while (i < size) {
+                    char c = s.charAt(i++);
+                    if (c == ',') {
+                        break;
+                    } else if (c == '\"') {
+                        while (i < size) {
+                            c = s.charAt(i++);
+                            if (c == '\\') {
+                                if (i++ == size) {
+                                    throw newMVStoreException(ERROR_FILE_CORRUPT, "Not a map: {0}", s);
+                                }
+                            } else if (c == '\"') {
+                                break;
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        return null;
+    }
+
     /**
      * Calculate the Fletcher32 checksum.
      *
      * @param bytes the bytes
+     * @param offset initial offset
      * @param length the message length (if odd, 0 is appended)
      * @return the checksum
      */
-    public static int getFletcher32(byte[] bytes, int length) {
+    public static int getFletcher32(byte[] bytes, int offset, int length) {
         int s1 = 0xffff, s2 = 0xffff;
-        int i = 0, evenLength = length / 2 * 2;
-        while (i < evenLength) {
+        int i = offset, len = offset + (length & ~1);
+        while (i < len) {
             // reduce after 360 words (each word is two bytes)
-            for (int end = Math.min(i + 720, evenLength); i < end;) {
+            for (int end = Math.min(i + 720, len); i < end;) {
                 int x = ((bytes[i++] & 0xff) << 8) | (bytes[i++] & 0xff);
                 s2 += s1 += x;
             }
             s1 = (s1 & 0xffff) + (s1 >>> 16);
             s2 = (s2 & 0xffff) + (s2 >>> 16);
         }
-        if (i < length) {
+        if ((length & 1) != 0) {
             // odd length: append 0
             int x = (bytes[i] & 0xff) << 8;
             s2 += s1 += x;
@@ -750,27 +983,16 @@ public class DataUtils {
     }
 
     /**
-     * Create a new ConcurrentModificationException.
-     *
-     * @param message the message
-     * @return the exception
-     */
-    public static ConcurrentModificationException
-            newConcurrentModificationException(String message) {
-        return new ConcurrentModificationException(formatMessage(0, message));
-    }
-
-    /**
-     * Create a new IllegalStateException.
+     * Create a new MVStoreException.
      *
      * @param errorCode the error code
      * @param message the message
      * @param arguments the arguments
      * @return the exception
      */
-    public static IllegalStateException newIllegalStateException(
+    public static MVStoreException newMVStoreException(
             int errorCode, String message, Object... arguments) {
-        return initCause(new IllegalStateException(
+        return initCause(new MVStoreException(errorCode,
                 formatMessage(errorCode, message, arguments)),
                 arguments);
     }
@@ -779,8 +1001,8 @@ public class DataUtils {
         int size = arguments.length;
         if (size > 0) {
             Object o = arguments[size - 1];
-            if (o instanceof Exception) {
-                e.initCause((Exception) o);
+            if (o instanceof Throwable) {
+                e.initCause((Throwable) o);
             }
         }
         return e;
@@ -797,6 +1019,7 @@ public class DataUtils {
     public static String formatMessage(int errorCode, String message,
             Object... arguments) {
         // convert arguments to strings, to avoid locale specific formatting
+        arguments = arguments.clone();
         for (int i = 0; i < arguments.length; i++) {
             Object a = arguments[i];
             if (!(a instanceof Exception)) {
@@ -813,79 +1036,6 @@ public class DataUtils {
                 "/" + errorCode + "]";
     }
 
-    /**
-     * Get the error code from an exception message.
-     *
-     * @param m the message
-     * @return the error code, or 0 if none
-     */
-    public static int getErrorCode(String m) {
-        if (m != null && m.endsWith("]")) {
-            int dash = m.lastIndexOf('/');
-            if (dash >= 0) {
-                String s = m.substring(dash + 1, m.length() - 1);
-                try {
-                    return Integer.parseInt(s);
-                } catch (NumberFormatException e) {
-                    // no error code
-                }
-            }
-        }
-        return 0;
-    }
-
-    /**
-     * Create an array of bytes with the given size. If this is not possible
-     * because not enough memory is available, an OutOfMemoryError with the
-     * requested size in the message is thrown.
-     * <p>
-     * This method should be used if the size of the array is user defined, or
-     * stored in a file, so wrong size data can be distinguished from regular
-     * out-of-memory.
-     *
-     * @param len the number of bytes requested
-     * @return the byte array
-     * @throws OutOfMemoryError if the allocation was too large
-     */
-    public static byte[] newBytes(int len) {
-        if (len == 0) {
-            return EMPTY_BYTES;
-        }
-        try {
-            return new byte[len];
-        } catch (OutOfMemoryError e) {
-            Error e2 = new OutOfMemoryError("Requested memory: " + len);
-            e2.initCause(e);
-            throw e2;
-        }
-    }
-
-    /**
-     * Ensure the byte buffer has the given capacity, plus 1 KB. If not, a new,
-     * larger byte buffer is created and the data is copied.
-     *
-     * @param buff the byte buffer
-     * @param len the minimum remaining capacity
-     * @return the byte buffer (possibly a new one)
-     */
-    public static ByteBuffer ensureCapacity(ByteBuffer buff, int len) {
-        len += 1024;
-        if (buff.remaining() > len) {
-            return buff;
-        }
-        return grow(buff, len);
-    }
-
-    private static ByteBuffer grow(ByteBuffer buff, int len) {
-        len = buff.remaining() + len;
-        int capacity = buff.capacity();
-        len = Math.max(len, Math.min(capacity + MAX_GROW, capacity * 2));
-        ByteBuffer temp = ByteBuffer.allocate(len);
-        buff.flip();
-        temp.put(buff);
-        return temp;
-    }
-
     /**
      * Read a hex long value from a map.
      *
@@ -893,10 +1043,9 @@ public class DataUtils {
      * @param key the key
      * @param defaultValue if the value is null
      * @return the parsed value
-     * @throws IllegalStateException if parsing fails
+     * @throws MVStoreException if parsing fails
      */
-    public static long readHexLong(Map<String, ? extends Object> map,
-            String key, long defaultValue) {
+    public static long readHexLong(Map<String, ?> map, String key, long defaultValue) {
         Object v = map.get(key);
         if (v == null) {
             return defaultValue;
@@ -906,7 +1055,7 @@ public class DataUtils {
         try {
             return parseHexLong((String) v);
         } catch (NumberFormatException e) {
-            throw newIllegalStateException(ERROR_FILE_CORRUPT,
+            throw newMVStoreException(ERROR_FILE_CORRUPT,
                     "Error parsing the value {0}", v, e);
         }
     }
@@ -916,7 +1065,7 @@ public class DataUtils {
      *
      * @param x the string
      * @return the parsed value
-     * @throws IllegalStateException if parsing fails
+     * @throws MVStoreException if parsing fails
      */
     public static long parseHexLong(String x) {
         try {
@@ -928,7 +1077,7 @@ public class DataUtils {
             }
             return Long.parseLong(x, 16);
         } catch (NumberFormatException e) {
-            throw newIllegalStateException(ERROR_FILE_CORRUPT,
+            throw newMVStoreException(ERROR_FILE_CORRUPT,
                     "Error parsing the value {0}", x, e);
         }
     }
@@ -938,7 +1087,7 @@ public class DataUtils {
      *
      * @param x the string
      * @return the parsed value
-     * @throws IllegalStateException if parsing fails
+     * @throws MVStoreException if parsing fails
      */
     public static int parseHexInt(String x) {
         try {
@@ -946,7 +1095,7 @@ public class DataUtils {
             // in Java 8, we can use Integer.parseLong(x, 16);
             return (int) Long.parseLong(x, 16);
         } catch (NumberFormatException e) {
-            throw newIllegalStateException(ERROR_FILE_CORRUPT,
+            throw newMVStoreException(ERROR_FILE_CORRUPT,
                     "Error parsing the value {0}", x, e);
         }
     }
@@ -958,10 +1107,9 @@ public class DataUtils {
      * @param key the key
      * @param defaultValue if the value is null
      * @return the parsed value
-     * @throws IllegalStateException if parsing fails
+     * @throws MVStoreException if parsing fails
      */
-    public static int readHexInt(HashMap<String, ? extends Object> map,
-            String key, int defaultValue) {
+    static int readHexInt(Map<String, ?> map, String key, int defaultValue) {
         Object v = map.get(key);
         if (v == null) {
             return defaultValue;
@@ -972,43 +1120,63 @@ public class DataUtils {
             // support unsigned hex value
             return (int) Long.parseLong((String) v, 16);
         } catch (NumberFormatException e) {
-            throw newIllegalStateException(ERROR_FILE_CORRUPT,
+            throw newMVStoreException(ERROR_FILE_CORRUPT,
                     "Error parsing the value {0}", v, e);
         }
     }
 
     /**
-     * An entry of a map.
+     * Parse the hex-encoded bytes of an entry in the map.
      *
-     * @param <K> the key type
-     * @param <V> the value type
+     * @param map the map
+     * @param key the key
+     * @return the byte array, or null if not in the map
      */
-    public static class MapEntry<K, V> implements Map.Entry<K, V> {
-
-        private final K key;
-        private V value;
-
-        public MapEntry(K key, V value) {
-            this.key = key;
-            this.value = value;
+    static byte[] parseHexBytes(Map<String, ?> map, String key) {
+        Object v = map.get(key);
+        if (v == null) {
+            return null;
         }
-
-        @Override
-        public K getKey() {
-            return key;
-        }
-
-        @Override
-        public V getValue() {
-            return value;
-        }
-
-        @Override
-        public V setValue(V value) {
-            throw DataUtils.newUnsupportedOperationException(
-                    "Updating the value is not supported");
-        }
-
+        return StringUtils.convertHexToBytes((String)v);
     }
 
+    /**
+     * Get the configuration parameter value, or default.
+     *
+     * @param config the configuration
+     * @param key the key
+     * @param defaultValue the default
+     * @return the configured value or default
+     */
+    static int getConfigParam(Map<String, ?> config, String key, int defaultValue) {
+        Object o = config.get(key);
+        if (o instanceof Number) {
+            return ((Number) o).intValue();
+        } else if (o != null) {
+            try {
+                return Integer.decode(o.toString());
+            } catch (NumberFormatException e) {
+                // ignore
+            }
+        }
+        return defaultValue;
+    }
+
+    /**
+     * Convert an exception to an IO exception.
+     *
+     * @param e the root cause
+     * @return the IO exception
+     */
+    public static IOException convertToIOException(Throwable e) {
+        if (e instanceof IOException) {
+            return (IOException) e;
+        }
+        if (e instanceof JdbcException) {
+            if (e.getCause() != null) {
+                e = e.getCause();
+            }
+        }
+        return new IOException(e.toString(), e);
+    }
 }
