@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -664,16 +664,17 @@ public final class Transaction {
      * @param toWaitFor transaction to wait for
      * @param mapName name of the map containing blocking entry
      * @param key of the blocking entry
+     * @param timeoutMillis timeout in milliseconds, {@code -1} for default
      * @return true if other transaction was closed and this one can proceed, false if timed out
      */
-    public boolean waitFor(Transaction toWaitFor, String mapName, Object key) {
+    public boolean waitFor(Transaction toWaitFor, String mapName, Object key, int timeoutMillis) {
         blockingTransaction = toWaitFor;
         blockingMapName = mapName;
         blockingKey = key;
         if (isDeadlocked(toWaitFor)) {
             tryThrowDeadLockException(false);
         }
-        boolean result = toWaitFor.waitForThisToEnd(timeoutMillis, this);
+        boolean result = toWaitFor.waitForThisToEnd(timeoutMillis == -1 ? this.timeoutMillis : timeoutMillis, this);
         blockingMapName = null;
         blockingKey = null;
         blockingTransaction = null;
