@@ -1,55 +1,56 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.schema;
 
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
+
 import org.h2.api.ErrorCode;
 import org.h2.command.ddl.CreateSynonymData;
 import org.h2.command.ddl.CreateTableData;
 import org.h2.constraint.Constraint;
 import org.h2.engine.Database;
 import org.h2.engine.DbObject;
-import org.h2.engine.DbObjectBase;
 import org.h2.engine.DbSettings;
-import org.h2.engine.FunctionAlias;
-import org.h2.engine.Session;
+import org.h2.engine.Right;
+import org.h2.engine.RightOwner;
+import org.h2.engine.SessionLocal;
 import org.h2.engine.SysProperties;
-import org.h2.engine.User;
 import org.h2.index.Index;
 import org.h2.message.DbException;
 import org.h2.message.Trace;
-import org.h2.mvstore.db.MVTableEngine;
-import org.h2.table.RegularTable;
+import org.h2.table.MaterializedView;
+import org.h2.table.MetaTable;
 import org.h2.table.Table;
 import org.h2.table.TableLink;
 import org.h2.table.TableSynonym;
-import org.h2.util.New;
-import org.h2.util.StringUtils;
+import org.h2.util.Utils;
 
 /**
  * A schema as created by the SQL statement
  * CREATE SCHEMA
  */
-public class Schema extends DbObjectBase {
+public class Schema extends DbObject {
 
-    private User owner;
+    private RightOwner owner;
     private final boolean system;
     private ArrayList<String> tableEngineParams;
 
     private final ConcurrentHashMap<String, Table> tablesAndViews;
+    private final ConcurrentHashMap<String, Domain> domains;
     private final ConcurrentHashMap<String, TableSynonym> synonyms;
     private final ConcurrentHashMap<String, Index> indexes;
     private final ConcurrentHashMap<String, Sequence> sequences;
     private final ConcurrentHashMap<String, TriggerObject> triggers;
     private final ConcurrentHashMap<String, Constraint> constraints;
     private final ConcurrentHashMap<String, Constant> constants;
-    private final ConcurrentHashMap<String, FunctionAlias> functions;
+    private final ConcurrentHashMap<String, UserDefinedFunction> functionsAndAggregates;
 
     /**
      * The set of returned unique names that are not yet stored. It is used to
@@ -68,17 +69,17 @@ public class Schema extends DbObjectBase {
      * @param system if this is a system schema (such a schema can not be
      *            dropped)
      */
-    public Schema(Database database, int id, String schemaName, User owner,
-            boolean system) {
+    public Schema(Database database, int id, String schemaName, RightOwner owner, boolean system) {
+        super(database, id, schemaName, Trace.SCHEMA);
         tablesAndViews = database.newConcurrentStringMap();
+        domains = database.newConcurrentStringMap();
         synonyms = database.newConcurrentStringMap();
         indexes = database.newConcurrentStringMap();
         sequences = database.newConcurrentStringMap();
         triggers = database.newConcurrentStringMap();
         constraints = database.newConcurrentStringMap();
         constants = database.newConcurrentStringMap();
-        functions = database.newConcurrentStringMap();
-        initDbObjectBase(database, id, schemaName, Trace.SCHEMA);
+        functionsAndAggregates = database.newConcurrentStringMap();
         this.owner = owner;
         this.system = system;
     }
@@ -92,23 +93,15 @@ public class Schema extends DbObjectBase {
         return !system;
     }
 
-    @Override
-    public String getCreateSQLForCopy(Table table, String quotedName) {
-        throw DbException.throwInternalError(toString());
-    }
-
-    @Override
-    public String getDropSQL() {
-        return null;
-    }
-
     @Override
     public String getCreateSQL() {
         if (system) {
             return null;
         }
-        return "CREATE SCHEMA IF NOT EXISTS " +
-            getSQL() + " AUTHORIZATION " + owner.getSQL();
+        StringBuilder builder = new StringBuilder("CREATE SCHEMA IF NOT EXISTS ");
+        getSQL(builder, DEFAULT_SQL_FLAGS).append(" AUTHORIZATION ");
+        owner.getSQL(builder, DEFAULT_SQL_FLAGS);
+        return builder.toString();
     }
 
     @Override
@@ -122,64 +115,82 @@ public class Schema extends DbObjectBase {
      * @return {@code true} if this schema is empty, {@code false} otherwise
      */
     public boolean isEmpty() {
-        return tablesAndViews.isEmpty() && synonyms.isEmpty() && indexes.isEmpty() && sequences.isEmpty()
-                && triggers.isEmpty() && constraints.isEmpty() && constants.isEmpty() && functions.isEmpty();
+        return tablesAndViews.isEmpty() && domains.isEmpty() && synonyms.isEmpty() && indexes.isEmpty()
+                && sequences.isEmpty() && triggers.isEmpty() && constraints.isEmpty() && constants.isEmpty()
+                && functionsAndAggregates.isEmpty();
     }
 
     @Override
-    public void removeChildrenAndResources(Session session) {
-        while (triggers != null && triggers.size() > 0) {
-            TriggerObject obj = (TriggerObject) triggers.values().toArray()[0];
-            database.removeSchemaObject(session, obj);
-        }
-        while (constraints != null && constraints.size() > 0) {
-            Constraint obj = (Constraint) constraints.values().toArray()[0];
-            database.removeSchemaObject(session, obj);
+    public ArrayList<DbObject> getChildren() {
+        ArrayList<DbObject> children = Utils.newSmallArrayList();
+        ArrayList<Right> rights = database.getAllRights();
+        for (Right right : rights) {
+            if (right.getGrantedObject() == this) {
+                children.add(right);
+            }
         }
+        return children;
+    }
+
+    @Override
+    public void removeChildrenAndResources(SessionLocal session) {
+        removeChildrenFromMap(session, triggers);
+        removeChildrenFromMap(session, constraints);
         // There can be dependencies between tables e.g. using computed columns,
         // so we might need to loop over them multiple times.
-        boolean runLoopAgain = false;
-        do {
-            runLoopAgain = false;
-            if (tablesAndViews != null) {
-                // Loop over a copy because the map is modified underneath us.
-                for (Table obj : new ArrayList<>(tablesAndViews.values())) {
-                    // Check for null because multiple tables might be deleted
-                    // in one go underneath us.
-                    if (obj.getName() != null) {
-                        if (database.getDependentTable(obj, obj) == null) {
-                            database.removeSchemaObject(session, obj);
-                        } else {
-                            runLoopAgain = true;
-                        }
+        boolean modified = true;
+        while (!tablesAndViews.isEmpty()) {
+            boolean newModified = false;
+            for (Table obj : tablesAndViews.values()) {
+                if (obj.getName() != null) {
+                    // Database.removeSchemaObject() removes the object from
+                    // the map too, but it is safe for ConcurrentHashMap.
+                    Table dependentTable = database.getDependentTable(obj, obj);
+                    if (dependentTable == null) {
+                        database.removeSchemaObject(session, obj);
+                        newModified = true;
+                    } else if (dependentTable.getSchema() != this) {
+                        throw DbException.get(ErrorCode.CANNOT_DROP_2, //
+                                obj.getTraceSQL(), dependentTable.getTraceSQL());
+                    } else if (!modified) {
+                        dependentTable.removeColumnExpressionsDependencies(session);
+                        dependentTable.setModified();
+                        database.updateMeta(session, dependentTable);
                     }
                 }
             }
-        } while (runLoopAgain);
-        while (indexes != null && indexes.size() > 0) {
-            Index obj = (Index) indexes.values().toArray()[0];
-            database.removeSchemaObject(session, obj);
+            modified = newModified;
         }
-        while (sequences != null && sequences.size() > 0) {
-            Sequence obj = (Sequence) sequences.values().toArray()[0];
-            database.removeSchemaObject(session, obj);
-        }
-        while (constants != null && constants.size() > 0) {
-            Constant obj = (Constant) constants.values().toArray()[0];
-            database.removeSchemaObject(session, obj);
-        }
-        while (functions != null && functions.size() > 0) {
-            FunctionAlias obj = (FunctionAlias) functions.values().toArray()[0];
-            database.removeSchemaObject(session, obj);
+        removeChildrenFromMap(session, domains);
+        removeChildrenFromMap(session, indexes);
+        removeChildrenFromMap(session, sequences);
+        removeChildrenFromMap(session, constants);
+        removeChildrenFromMap(session, functionsAndAggregates);
+        for (Right right : database.getAllRights()) {
+            if (right.getGrantedObject() == this) {
+                database.removeDatabaseObject(session, right);
+            }
         }
         database.removeMeta(session, getId());
         owner = null;
         invalidate();
     }
 
-    @Override
-    public void checkRename() {
-        // ok
+    private void removeChildrenFromMap(SessionLocal session, ConcurrentHashMap<String, ? extends SchemaObject> map) {
+        if (!map.isEmpty()) {
+            for (SchemaObject obj : map.values()) {
+                /*
+                 * Referential constraints are dropped when unique or PK
+                 * constraint is dropped, but iterator may return already
+                 * removed objects in some cases.
+                 */
+                if (obj.isValid()) {
+                    // Database.removeSchemaObject() removes the object from
+                    // the map too, but it is safe for ConcurrentHashMap.
+                    database.removeSchemaObject(session, obj);
+                }
+            }
+        }
     }
 
     /**
@@ -187,7 +198,7 @@ public class Schema extends DbObjectBase {
      *
      * @return the owner
      */
-    public User getOwner() {
+    public RightOwner getOwner() {
         return owner;
     }
 
@@ -215,6 +226,9 @@ public class Schema extends DbObjectBase {
         case DbObject.TABLE_OR_VIEW:
             result = tablesAndViews;
             break;
+        case DbObject.DOMAIN:
+            result = domains;
+            break;
         case DbObject.SYNONYM:
             result = synonyms;
             break;
@@ -234,10 +248,11 @@ public class Schema extends DbObjectBase {
             result = constants;
             break;
         case DbObject.FUNCTION_ALIAS:
-            result = functions;
+        case DbObject.AGGREGATE:
+            result = functionsAndAggregates;
             break;
         default:
-            throw DbException.throwInternalError("type=" + type);
+            throw DbException.getInternalError("type=" + type);
         }
         return (Map<String, SchemaObject>) result;
     }
@@ -250,15 +265,14 @@ public class Schema extends DbObjectBase {
      * @param obj the object to add
      */
     public void add(SchemaObject obj) {
-        if (SysProperties.CHECK && obj.getSchema() != this) {
-            DbException.throwInternalError("wrong schema");
+        if (obj.getSchema() != this) {
+            throw DbException.getInternalError("wrong schema");
         }
         String name = obj.getName();
         Map<String, SchemaObject> map = getMap(obj.getType());
-        if (SysProperties.CHECK && map.get(name) != null) {
-            DbException.throwInternalError("object already exists: " + name);
+        if (map.putIfAbsent(name, obj) != null) {
+            throw DbException.getInternalError("object already exists: " + name);
         }
-        map.put(name, obj);
         freeUniqueName(name);
     }
 
@@ -272,11 +286,11 @@ public class Schema extends DbObjectBase {
         int type = obj.getType();
         Map<String, SchemaObject> map = getMap(type);
         if (SysProperties.CHECK) {
-            if (!map.containsKey(obj.getName())) {
-                DbException.throwInternalError("not found: " + obj.getName());
+            if (!map.containsKey(obj.getName()) && !(obj instanceof MetaTable)) {
+                throw DbException.getInternalError("not found: " + obj.getName());
             }
             if (obj.getName().equals(newName) || map.containsKey(newName)) {
-                DbException.throwInternalError("object already exists: " + newName);
+                throw DbException.getInternalError("object already exists: " + newName);
             }
         }
         obj.checkRename();
@@ -296,7 +310,7 @@ public class Schema extends DbObjectBase {
      * @param name the object name
      * @return the object or null
      */
-    public Table findTableOrView(Session session, String name) {
+    public Table findTableOrView(SessionLocal session, String name) {
         Table table = tablesAndViews.get(name);
         if (table == null && session != null) {
             table = session.findLocalTempTable(name);
@@ -314,14 +328,33 @@ public class Schema extends DbObjectBase {
      * @param name the object name
      * @return the object or null
      */
-    public Table resolveTableOrView(Session session, String name) {
+    public Table resolveTableOrView(SessionLocal session, String name) {
+        return resolveTableOrView(session, name, /*resolveMaterializedView*/true);
+    }
+
+    /**
+     * Try to find a table or view with this name. This method returns null if
+     * no object with this name exists. Local temporary tables are also
+     * returned. If a synonym with this name exists, the backing table of the
+     * synonym is returned
+     *
+     * @param session the session
+     * @param name the object name
+     * @param resolveMaterializedView if true, and the object is a materialized
+     *             view, return the underlying Table object.
+     * @return the object or null
+     */
+    public Table resolveTableOrView(SessionLocal session, String name, boolean resolveMaterializedView) {
         Table table = findTableOrView(session, name);
         if (table == null) {
             TableSynonym synonym = synonyms.get(name);
             if (synonym != null) {
                 return synonym.getSynonymFor();
             }
-            return null;
+        }
+        if (resolveMaterializedView && table instanceof MaterializedView) {
+            MaterializedView matView = (MaterializedView) table;
+            return matView.getUnderlyingTable();
         }
         return table;
     }
@@ -337,6 +370,16 @@ public class Schema extends DbObjectBase {
         return synonyms.get(name);
     }
 
+    /**
+     * Get the domain if it exists, or null if not.
+     *
+     * @param name the name of the domain
+     * @return the domain or null
+     */
+    public Domain findDomain(String name) {
+        return domains.get(name);
+    }
+
     /**
      * Try to find an index with this name. This method returns null if
      * no object with this name exists.
@@ -345,7 +388,7 @@ public class Schema extends DbObjectBase {
      * @param name the object name
      * @return the object or null
      */
-    public Index findIndex(Session session, String name) {
+    public Index findIndex(SessionLocal session, String name) {
         Index index = indexes.get(name);
         if (index == null) {
             index = session.findLocalTempTableIndex(name);
@@ -383,7 +426,7 @@ public class Schema extends DbObjectBase {
      * @param name the object name
      * @return the object or null
      */
-    public Constraint findConstraint(Session session, String name) {
+    public Constraint findConstraint(SessionLocal session, String name) {
         Constraint constraint = constraints.get(name);
         if (constraint == null) {
             constraint = session.findLocalTempTableConstraint(name);
@@ -410,7 +453,46 @@ public class Schema extends DbObjectBase {
      * @return the object or null
      */
     public FunctionAlias findFunction(String functionAlias) {
-        return functions.get(functionAlias);
+        UserDefinedFunction userDefinedFunction = findFunctionOrAggregate(functionAlias);
+        return userDefinedFunction instanceof FunctionAlias ? (FunctionAlias) userDefinedFunction : null;
+    }
+
+    /**
+     * Get the user defined aggregate function if it exists. This method returns
+     * null if no object with this name exists.
+     *
+     * @param name the name of the user defined aggregate function
+     * @return the aggregate function or null
+     */
+    public UserAggregate findAggregate(String name) {
+        UserDefinedFunction userDefinedFunction = findFunctionOrAggregate(name);
+        return userDefinedFunction instanceof UserAggregate ? (UserAggregate) userDefinedFunction : null;
+    }
+
+    /**
+     * Try to find a user defined function or aggregate function with the
+     * specified name. This method returns null if no object with this name
+     * exists.
+     *
+     * @param name
+     *            the object name
+     * @return the object or null
+     */
+    public UserDefinedFunction findFunctionOrAggregate(String name) {
+        return functionsAndAggregates.get(name);
+    }
+
+    /**
+     * Reserve a unique object name.
+     *
+     * @param name the object name
+     */
+    public void reserveUniqueName(String name) {
+        if (name != null) {
+            synchronized (temporaryUniqueNames) {
+                temporaryUniqueNames.add(name);
+            }
+        }
     }
 
     /**
@@ -426,30 +508,26 @@ public class Schema extends DbObjectBase {
         }
     }
 
-    private String getUniqueName(DbObject obj,
-            Map<String, ? extends SchemaObject> map, String prefix) {
-        String hash = StringUtils.toUpperEnglish(Integer.toHexString(obj.getName().hashCode()));
-        String name = null;
+    private String getUniqueName(DbObject obj, Map<String, ? extends SchemaObject> map, String prefix) {
+        StringBuilder nameBuilder = new StringBuilder(prefix);
+        String hash = Integer.toHexString(obj.getName().hashCode());
         synchronized (temporaryUniqueNames) {
-            for (int i = 1, len = hash.length(); i < len; i++) {
-                name = prefix + hash.substring(0, i);
-                if (!map.containsKey(name) && !temporaryUniqueNames.contains(name)) {
-                    break;
-                }
-                name = null;
-            }
-            if (name == null) {
-                prefix = prefix + hash + "_";
-                for (int i = 0;; i++) {
-                    name = prefix + i;
-                    if (!map.containsKey(name) && !temporaryUniqueNames.contains(name)) {
-                        break;
-                    }
+            for (int i = 0, len = hash.length(); i < len; i++) {
+                char c = hash.charAt(i);
+                String name = nameBuilder.append(c >= 'a' ? (char) (c - 0x20) : c).toString();
+                if (!map.containsKey(name) && temporaryUniqueNames.add(name)) {
+                    return name;
                 }
             }
-            temporaryUniqueNames.add(name);
+            int nameLength = nameBuilder.append('_').length();
+            for (int i = 0;; i++) {
+                String name = nameBuilder.append(i).toString();
+                if (!map.containsKey(name) && temporaryUniqueNames.add(name)) {
+                    return name;
+                }
+                nameBuilder.setLength(nameLength);
+            }
         }
-        return name;
     }
 
     /**
@@ -459,7 +537,7 @@ public class Schema extends DbObjectBase {
      * @param table the constraint table
      * @return the unique name
      */
-    public String getUniqueConstraintName(Session session, Table table) {
+    public String getUniqueConstraintName(SessionLocal session, Table table) {
         Map<String, Constraint> tableConstraints;
         if (table.isTemporary() && !table.isGlobalTemporary()) {
             tableConstraints = session.getLocalTempTableConstraints();
@@ -469,6 +547,17 @@ public class Schema extends DbObjectBase {
         return getUniqueName(table, tableConstraints, "CONSTRAINT_");
     }
 
+    /**
+     * Create a unique constraint name.
+     *
+     * @param session the session
+     * @param domain the constraint domain
+     * @return the unique name
+     */
+    public String getUniqueDomainConstraintName(SessionLocal session, Domain domain) {
+        return getUniqueName(domain, constraints, "CONSTRAINT_");
+    }
+
     /**
      * Create a unique index name.
      *
@@ -477,7 +566,7 @@ public class Schema extends DbObjectBase {
      * @param prefix the index name prefix
      * @return the unique name
      */
-    public String getUniqueIndexName(Session session, Table table, String prefix) {
+    public String getUniqueIndexName(SessionLocal session, Table table, String prefix) {
         Map<String, Index> tableIndexes;
         if (table.isTemporary() && !table.isGlobalTemporary()) {
             tableIndexes = session.getLocalTempTableIndexes();
@@ -496,7 +585,7 @@ public class Schema extends DbObjectBase {
      * @return the table or view
      * @throws DbException if no such object exists
      */
-    public Table getTableOrView(Session session, String name) {
+    public Table getTableOrView(SessionLocal session, String name) {
         Table table = tablesAndViews.get(name);
         if (table == null) {
             if (session != null) {
@@ -509,6 +598,21 @@ public class Schema extends DbObjectBase {
         return table;
     }
 
+    /**
+     * Get the domain with the given name.
+     *
+     * @param name the domain name
+     * @return the domain
+     * @throws DbException if no such object exists
+     */
+    public Domain getDomain(String name) {
+        Domain domain = domains.get(name);
+        if (domain == null) {
+            throw DbException.get(ErrorCode.DOMAIN_NOT_FOUND_1, name);
+        }
+        return domain;
+    }
+
     /**
      * Get the index with the given name.
      *
@@ -572,60 +676,91 @@ public class Schema extends DbObjectBase {
     /**
      * Get all objects.
      *
-     * @return a (possible empty) list of all objects
+     * @param addTo
+     *                  list to add objects to, or {@code null} to allocate a new
+     *                  list
+     * @return the specified list with added objects, or a new (possibly empty) list
+     *         with all objects
      */
-    public ArrayList<SchemaObject> getAll() {
-        ArrayList<SchemaObject> all = New.arrayList();
-        all.addAll(getMap(DbObject.TABLE_OR_VIEW).values());
-        all.addAll(getMap(DbObject.SYNONYM).values());
-        all.addAll(getMap(DbObject.SEQUENCE).values());
-        all.addAll(getMap(DbObject.INDEX).values());
-        all.addAll(getMap(DbObject.TRIGGER).values());
-        all.addAll(getMap(DbObject.CONSTRAINT).values());
-        all.addAll(getMap(DbObject.CONSTANT).values());
-        all.addAll(getMap(DbObject.FUNCTION_ALIAS).values());
-        return all;
+    public ArrayList<SchemaObject> getAll(ArrayList<SchemaObject> addTo) {
+        if (addTo == null) {
+            addTo = Utils.newSmallArrayList();
+        }
+        addTo.addAll(tablesAndViews.values());
+        addTo.addAll(domains.values());
+        addTo.addAll(synonyms.values());
+        addTo.addAll(sequences.values());
+        addTo.addAll(indexes.values());
+        addTo.addAll(triggers.values());
+        addTo.addAll(constraints.values());
+        addTo.addAll(constants.values());
+        addTo.addAll(functionsAndAggregates.values());
+        return addTo;
     }
 
     /**
      * Get all objects of the given type.
      *
-     * @param type the object type
-     * @return a (possible empty) list of all objects
+     * @param type
+     *                  the object type
+     * @param addTo
+     *                  list to add objects to
      */
-    public ArrayList<SchemaObject> getAll(int type) {
-        Map<String, SchemaObject> map = getMap(type);
-        return new ArrayList<>(map.values());
+    public void getAll(int type, ArrayList<SchemaObject> addTo) {
+        addTo.addAll(getMap(type).values());
+    }
+
+    public Collection<Domain> getAllDomains() {
+        return domains.values();
+    }
+
+    public Collection<Constraint> getAllConstraints() {
+        return constraints.values();
+    }
+
+    public Collection<Constant> getAllConstants() {
+        return constants.values();
+    }
+
+    public Collection<Sequence> getAllSequences() {
+        return sequences.values();
+    }
+
+    public Collection<TriggerObject> getAllTriggers() {
+        return triggers.values();
     }
 
     /**
      * Get all tables and views.
      *
+     * @param session the session, {@code null} to exclude meta tables
      * @return a (possible empty) list of all objects
      */
-    public ArrayList<Table> getAllTablesAndViews() {
-        synchronized (database) {
-            return new ArrayList<>(tablesAndViews.values());
-        }
+    public Collection<Table> getAllTablesAndViews(SessionLocal session) {
+        return tablesAndViews.values();
     }
 
+    public Collection<Index> getAllIndexes() {
+        return indexes.values();
+    }
 
-    public ArrayList<TableSynonym> getAllSynonyms() {
-        synchronized (database) {
-            return new ArrayList<>(synonyms.values());
-        }
+    public Collection<TableSynonym> getAllSynonyms() {
+        return synonyms.values();
+    }
+
+    public Collection<UserDefinedFunction> getAllFunctionsAndAggregates() {
+        return functionsAndAggregates.values();
     }
 
     /**
      * Get the table with the given name, if any.
      *
+     * @param session the session
      * @param name the table name
      * @return the table or null if not found
      */
-    public Table getTableOrViewByName(String name) {
-        synchronized (database) {
-            return tablesAndViews.get(name);
-        }
+    public Table getTableOrViewByName(SessionLocal session, String name) {
+        return tablesAndViews.get(name);
     }
 
     /**
@@ -636,10 +771,9 @@ public class Schema extends DbObjectBase {
     public void remove(SchemaObject obj) {
         String objName = obj.getName();
         Map<String, SchemaObject> map = getMap(obj.getType());
-        if (SysProperties.CHECK && !map.containsKey(objName)) {
-            DbException.throwInternalError("not found: " + objName);
+        if (map.remove(objName) == null) {
+            throw DbException.getInternalError("not found: " + objName);
         }
-        map.remove(objName);
         freeUniqueName(objName);
     }
 
@@ -655,21 +789,19 @@ public class Schema extends DbObjectBase {
                 database.lockMeta(data.session);
             }
             data.schema = this;
-            if (data.tableEngine == null) {
+            String tableEngine = data.tableEngine;
+            if (tableEngine == null) {
                 DbSettings s = database.getSettings();
-                if (s.defaultTableEngine != null) {
-                    data.tableEngine = s.defaultTableEngine;
-                } else if (s.mvStore) {
-                    data.tableEngine = MVTableEngine.class.getName();
+                tableEngine = s.defaultTableEngine;
+                if (tableEngine == null) {
+                    return database.getStore().createTable(data);
                 }
+                data.tableEngine = tableEngine;
             }
-            if (data.tableEngine != null) {
-                if (data.tableEngineParams == null) {
-                    data.tableEngineParams = this.tableEngineParams;
-                }
-                return database.getTableEngine(data.tableEngine).createTable(data);
+            if (data.tableEngineParams == null) {
+                data.tableEngineParams = this.tableEngineParams;
             }
-            return new RegularTable(data);
+            return database.getTableEngine(tableEngine).createTable(data);
         }
     }
 
