@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.server;
@@ -14,29 +14,37 @@ import java.io.StringWriter;
 import java.net.Socket;
 import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.Objects;
+
 import org.h2.api.ErrorCode;
 import org.h2.command.Command;
 import org.h2.engine.ConnectionInfo;
 import org.h2.engine.Constants;
 import org.h2.engine.Engine;
+import org.h2.engine.GeneratedKeysMode;
 import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.engine.SessionRemote;
 import org.h2.engine.SysProperties;
 import org.h2.expression.Parameter;
 import org.h2.expression.ParameterInterface;
 import org.h2.expression.ParameterRemote;
-import org.h2.jdbc.JdbcSQLException;
+import org.h2.jdbc.JdbcException;
+import org.h2.jdbc.meta.DatabaseMetaServer;
 import org.h2.message.DbException;
 import org.h2.result.ResultColumn;
 import org.h2.result.ResultInterface;
+import org.h2.result.ResultWithGeneratedKeys;
 import org.h2.store.LobStorageInterface;
 import org.h2.util.IOUtils;
+import org.h2.util.NetUtils;
+import org.h2.util.NetworkConnectionInfo;
 import org.h2.util.SmallLRUCache;
 import org.h2.util.SmallMap;
-import org.h2.util.StringUtils;
+import org.h2.util.TimeZoneProvider;
 import org.h2.value.Transfer;
 import org.h2.value.Value;
-import org.h2.value.ValueLobDb;
+import org.h2.value.ValueLob;
 
 /**
  * One server thread is opened per client connection.
@@ -45,7 +53,7 @@ public class TcpServerThread implements Runnable {
 
     protected final Transfer transfer;
     private final TcpServer server;
-    private Session session;
+    private SessionLocal session;
     private boolean stop;
     private Thread thread;
     private Command commit;
@@ -58,12 +66,12 @@ public class TcpServerThread implements Runnable {
     private final int threadId;
     private int clientVersion;
     private String sessionId;
+    private long lastRemoteSettingsId;
 
     TcpServerThread(Socket socket, TcpServer server, int id) {
         this.server = server;
         this.threadId = id;
-        transfer = new Transfer(null);
-        transfer.setSocket(socket);
+        transfer = new Transfer(null, socket);
     }
 
     private void trace(String s) {
@@ -78,22 +86,31 @@ public class TcpServerThread implements Runnable {
             // TODO server: should support a list of allowed databases
             // and a list of allowed clients
             try {
+                Socket socket = transfer.getSocket();
+                if (socket == null) {
+                    // the transfer is already closed, prevent NPE in TcpServer#allow(Socket)
+                    return;
+                }
                 if (!server.allow(transfer.getSocket())) {
                     throw DbException.get(ErrorCode.REMOTE_CONNECTION_NOT_ALLOWED);
                 }
                 int minClientVersion = transfer.readInt();
-                if (minClientVersion < Constants.TCP_PROTOCOL_VERSION_6) {
+                if (minClientVersion < 6) {
                     throw DbException.get(ErrorCode.DRIVER_VERSION_ERROR_2,
-                            "" + clientVersion, "" + Constants.TCP_PROTOCOL_VERSION_6);
-                } else if (minClientVersion > Constants.TCP_PROTOCOL_VERSION_16) {
-                    throw DbException.get(ErrorCode.DRIVER_VERSION_ERROR_2,
-                            "" + clientVersion, "" + Constants.TCP_PROTOCOL_VERSION_16);
+                            Integer.toString(minClientVersion), "" + Constants.TCP_PROTOCOL_VERSION_MIN_SUPPORTED);
                 }
                 int maxClientVersion = transfer.readInt();
-                if (maxClientVersion >= Constants.TCP_PROTOCOL_VERSION_16) {
-                    clientVersion = Constants.TCP_PROTOCOL_VERSION_16;
+                if (maxClientVersion < Constants.TCP_PROTOCOL_VERSION_MIN_SUPPORTED) {
+                    throw DbException.get(ErrorCode.DRIVER_VERSION_ERROR_2,
+                            Integer.toString(maxClientVersion), "" + Constants.TCP_PROTOCOL_VERSION_MIN_SUPPORTED);
+                } else if (minClientVersion > Constants.TCP_PROTOCOL_VERSION_MAX_SUPPORTED) {
+                    throw DbException.get(ErrorCode.DRIVER_VERSION_ERROR_2,
+                            Integer.toString(minClientVersion), "" + Constants.TCP_PROTOCOL_VERSION_MAX_SUPPORTED);
+                }
+                if (maxClientVersion >= Constants.TCP_PROTOCOL_VERSION_MAX_SUPPORTED) {
+                    clientVersion = Constants.TCP_PROTOCOL_VERSION_MAX_SUPPORTED;
                 } else {
-                    clientVersion = minClientVersion;
+                    clientVersion = maxClientVersion;
                 }
                 transfer.setVersion(clientVersion);
                 String db = transfer.readString();
@@ -135,29 +152,45 @@ public class TcpServerThread implements Runnable {
                     ci.setBaseDir(baseDir);
                 }
                 if (server.getIfExists()) {
-                    ci.setProperty("IFEXISTS", "TRUE");
+                    ci.setProperty("FORBID_CREATION", "TRUE");
                 }
                 transfer.writeInt(SessionRemote.STATUS_OK);
                 transfer.writeInt(clientVersion);
                 transfer.flush();
-                if (clientVersion >= Constants.TCP_PROTOCOL_VERSION_13) {
-                    if (ci.getFilePasswordHash() != null) {
-                        ci.setFileEncryptionKey(transfer.readBytes());
-                    }
+                if (ci.getFilePasswordHash() != null) {
+                    ci.setFileEncryptionKey(transfer.readBytes());
                 }
-                session = Engine.getInstance().createSession(ci);
+                ci.setNetworkConnectionInfo(new NetworkConnectionInfo(
+                        NetUtils.ipToShortForm(new StringBuilder(server.getSSL() ? "ssl://" : "tcp://"),
+                                socket.getLocalAddress().getAddress(), true) //
+                                .append(':').append(socket.getLocalPort()).toString(), //
+                        socket.getInetAddress().getAddress(), socket.getPort(),
+                        new StringBuilder().append('P').append(clientVersion).toString()));
+                if (clientVersion < Constants.TCP_PROTOCOL_VERSION_20) {
+                    // For DatabaseMetaData
+                    ci.setProperty("OLD_INFORMATION_SCHEMA", "TRUE");
+                    // For H2 Console
+                    ci.setProperty("NON_KEYWORDS", "VALUE");
+                }
+                session = Engine.createSession(ci);
                 transfer.setSession(session);
                 server.addConnection(threadId, originalURL, ci.getUserName());
                 trace("Connected");
+                lastRemoteSettingsId = session.getDatabase().getRemoteSettingsId();
+            } catch (OutOfMemoryError e) {
+                // catch this separately otherwise such errors will never hit the console
+                server.traceError(e);
+                sendError(e, true);
+                stop = true;
             } catch (Throwable e) {
-                sendError(e);
+                sendError(e,true);
                 stop = true;
             }
             while (!stop) {
                 try {
                     process();
                 } catch (Throwable e) {
-                    sendError(e);
+                    sendError(e, true);
                 }
             }
             trace("Disconnect");
@@ -171,23 +204,12 @@ public class TcpServerThread implements Runnable {
     private void closeSession() {
         if (session != null) {
             RuntimeException closeError = null;
-            try {
-                Command rollback = session.prepareLocal("ROLLBACK");
-                rollback.executeUpdate();
-            } catch (RuntimeException e) {
-                closeError = e;
-                server.traceError(e);
-            } catch (Exception e) {
-                server.traceError(e);
-            }
             try {
                 session.close();
                 server.removeConnection(threadId);
             } catch (RuntimeException e) {
-                if (closeError == null) {
-                    closeError = e;
-                    server.traceError(e);
-                }
+                closeError = e;
+                server.traceError(e);
             } catch (Exception e) {
                 server.traceError(e);
             } finally {
@@ -215,7 +237,7 @@ public class TcpServerThread implements Runnable {
         }
     }
 
-    private void sendError(Throwable t) {
+    private void sendError(Throwable t, boolean withStatus) {
         try {
             SQLException e = DbException.convert(t).getSQLException();
             StringWriter writer = new StringWriter();
@@ -223,15 +245,18 @@ public class TcpServerThread implements Runnable {
             String trace = writer.toString();
             String message;
             String sql;
-            if (e instanceof JdbcSQLException) {
-                JdbcSQLException j = (JdbcSQLException) e;
+            if (e instanceof JdbcException) {
+                JdbcException j = (JdbcException) e;
                 message = j.getOriginalMessage();
                 sql = j.getSQL();
             } else {
                 message = e.getMessage();
                 sql = null;
             }
-            transfer.writeInt(SessionRemote.STATUS_ERROR).
+            if (withStatus) {
+                transfer.writeInt(SessionRemote.STATUS_ERROR);
+            }
+            transfer.
                     writeString(e.getSQLState()).writeString(message).
                     writeString(sql).writeInt(e.getErrorCode()).writeString(trace).flush();
         } catch (Exception e2) {
@@ -248,16 +273,15 @@ public class TcpServerThread implements Runnable {
         ArrayList<? extends ParameterInterface> params = command.getParameters();
         for (int i = 0; i < len; i++) {
             Parameter p = (Parameter) params.get(i);
-            p.setValue(transfer.readValue());
+            p.setValue(transfer.readValue(null));
         }
     }
 
     private void process() throws IOException {
         int operation = transfer.readInt();
         switch (operation) {
-        case SessionRemote.SESSION_PREPARE_READ_PARAMS:
-        case SessionRemote.SESSION_PREPARE_READ_PARAMS2:
-        case SessionRemote.SESSION_PREPARE: {
+        case SessionRemote.SESSION_PREPARE:
+        case SessionRemote.SESSION_PREPARE_READ_PARAMS2: {
             int id = transfer.readInt();
             String sql = transfer.readString();
             int old = session.getModificationId();
@@ -269,7 +293,7 @@ public class TcpServerThread implements Runnable {
             transfer.writeInt(getState(old)).writeBoolean(isQuery).
                     writeBoolean(readonly);
 
-            if (operation == SessionRemote.SESSION_PREPARE_READ_PARAMS2) {
+            if (operation != SessionRemote.SESSION_PREPARE) {
                 transfer.writeInt(command.getCommandType());
             }
 
@@ -297,7 +321,7 @@ public class TcpServerThread implements Runnable {
                 commit = session.prepareLocal("COMMIT");
             }
             int old = session.getModificationId();
-            commit.executeUpdate();
+            commit.executeUpdate(null);
             transfer.writeInt(getState(old)).flush();
             break;
         }
@@ -309,7 +333,7 @@ public class TcpServerThread implements Runnable {
             cache.addObject(objectId, result);
             int columnCount = result.getVisibleColumnCount();
             transfer.writeInt(SessionRemote.STATUS_OK).
-                    writeInt(columnCount).writeInt(0);
+                    writeInt(columnCount).writeRowCount(0L);
             for (int i = 0; i < columnCount; i++) {
                 ResultColumn.writeColumn(transfer, result, i);
             }
@@ -319,7 +343,7 @@ public class TcpServerThread implements Runnable {
         case SessionRemote.COMMAND_EXECUTE_QUERY: {
             int id = transfer.readInt();
             int objectId = transfer.readInt();
-            int maxRows = transfer.readInt();
+            long maxRows = transfer.readRowCount();
             int fetchSize = transfer.readInt();
             Command command = (Command) cache.getObject(id, false);
             setParameters(command);
@@ -332,15 +356,12 @@ public class TcpServerThread implements Runnable {
             int columnCount = result.getVisibleColumnCount();
             int state = getState(old);
             transfer.writeInt(state).writeInt(columnCount);
-            int rowCount = result.getRowCount();
-            transfer.writeInt(rowCount);
+            long rowCount = result.isLazy() ? -1L : result.getRowCount();
+            transfer.writeRowCount(rowCount);
             for (int i = 0; i < columnCount; i++) {
                 ResultColumn.writeColumn(transfer, result, i);
             }
-            int fetch = Math.min(rowCount, fetchSize);
-            for (int i = 0; i < fetch; i++) {
-                sendRow(result);
-            }
+            sendRows(result, rowCount >= 0L ? Math.min(rowCount, fetchSize) : fetchSize);
             transfer.flush();
             break;
         }
@@ -348,10 +369,43 @@ public class TcpServerThread implements Runnable {
             int id = transfer.readInt();
             Command command = (Command) cache.getObject(id, false);
             setParameters(command);
+            boolean writeGeneratedKeys = true;
+            Object generatedKeysRequest;
+            int mode = transfer.readInt();
+            switch (mode) {
+            case GeneratedKeysMode.NONE:
+                generatedKeysRequest = false;
+                writeGeneratedKeys = false;
+                break;
+            case GeneratedKeysMode.AUTO:
+                generatedKeysRequest = true;
+                break;
+            case GeneratedKeysMode.COLUMN_NUMBERS: {
+                int len = transfer.readInt();
+                int[] keys = new int[len];
+                for (int i = 0; i < len; i++) {
+                    keys[i] = transfer.readInt();
+                }
+                generatedKeysRequest = keys;
+                break;
+            }
+            case GeneratedKeysMode.COLUMN_NAMES: {
+                int len = transfer.readInt();
+                String[] keys = new String[len];
+                for (int i = 0; i < len; i++) {
+                    keys[i] = transfer.readString();
+                }
+                generatedKeysRequest = keys;
+                break;
+            }
+            default:
+                throw DbException.get(ErrorCode.CONNECTION_BROKEN_1,
+                        "Unsupported generated keys' mode " + mode);
+            }
             int old = session.getModificationId();
-            int updateCount;
+            ResultWithGeneratedKeys result;
             synchronized (session) {
-                updateCount = command.executeUpdate();
+                result = command.executeUpdate(generatedKeysRequest);
             }
             int status;
             if (session.isClosed()) {
@@ -360,8 +414,21 @@ public class TcpServerThread implements Runnable {
             } else {
                 status = getState(old);
             }
-            transfer.writeInt(status).writeInt(updateCount).
-                    writeBoolean(session.getAutoCommit());
+            transfer.writeInt(status);
+            transfer.writeRowCount(result.getUpdateCount());
+            transfer.writeBoolean(session.getAutoCommit());
+            if (writeGeneratedKeys) {
+                ResultInterface generatedKeys = result.getGeneratedKeys();
+                int columnCount = generatedKeys.getVisibleColumnCount();
+                transfer.writeInt(columnCount);
+                long rowCount = generatedKeys.getRowCount();
+                transfer.writeRowCount(rowCount);
+                for (int i = 0; i < columnCount; i++) {
+                    ResultColumn.writeColumn(transfer, generatedKeys, i);
+                }
+                sendRows(generatedKeys, rowCount);
+                generatedKeys.close();
+            }
             transfer.flush();
             break;
         }
@@ -379,9 +446,7 @@ public class TcpServerThread implements Runnable {
             int count = transfer.readInt();
             ResultInterface result = (ResultInterface) cache.getObject(id, false);
             transfer.writeInt(SessionRemote.STATUS_OK);
-            for (int i = 0; i < count; i++) {
-                sendRow(result);
-            }
+            sendRows(result, count);
             transfer.flush();
             break;
         }
@@ -410,9 +475,12 @@ public class TcpServerThread implements Runnable {
         }
         case SessionRemote.SESSION_SET_ID: {
             sessionId = transfer.readString();
-            transfer.writeInt(SessionRemote.STATUS_OK);
-            transfer.writeBoolean(session.getAutoCommit());
-            transfer.flush();
+            if (clientVersion >= Constants.TCP_PROTOCOL_VERSION_20) {
+                session.setTimeZone(TimeZoneProvider.ofId(transfer.readString()));
+            }
+            transfer.writeInt(SessionRemote.STATUS_OK)
+                .writeBoolean(session.getAutoCommit())
+                .flush();
             break;
         }
         case SessionRemote.SESSION_SET_AUTOCOMMIT: {
@@ -428,40 +496,15 @@ public class TcpServerThread implements Runnable {
         }
         case SessionRemote.LOB_READ: {
             long lobId = transfer.readLong();
-            byte[] hmac;
-            CachedInputStream in;
-            boolean verifyMac;
-            if (clientVersion >= Constants.TCP_PROTOCOL_VERSION_11) {
-                if (clientVersion >= Constants.TCP_PROTOCOL_VERSION_12) {
-                    hmac = transfer.readBytes();
-                    verifyMac = true;
-                } else {
-                    hmac = null;
-                    verifyMac = false;
-                }
-                in = lobs.get(lobId);
-                if (in == null && verifyMac) {
-                    in = new CachedInputStream(null);
-                    lobs.put(lobId, in);
-                }
-            } else {
-                verifyMac = false;
-                hmac = null;
-                in = lobs.get(lobId);
-            }
+            byte[] hmac = transfer.readBytes();
             long offset = transfer.readLong();
             int length = transfer.readInt();
-            if (verifyMac) {
-                transfer.verifyLobMac(hmac, lobId);
-            }
-            if (in == null) {
-                throw DbException.get(ErrorCode.OBJECT_CLOSED);
-            }
-            if (in.getPos() != offset) {
+            transfer.verifyLobMac(hmac, lobId);
+            CachedInputStream in = lobs.get(lobId);
+            if (in == null || in.getPos() != offset) {
                 LobStorageInterface lobStorage = session.getDataHandler().getLobStorage();
                 // only the lob id is used
-                ValueLobDb lob = ValueLobDb.create(Value.BLOB, null, -1, lobId, hmac, -1);
-                InputStream lobIn = lobStorage.getInputStream(lob, hmac, -1);
+                InputStream lobIn = lobStorage.getInputStream(lobId, -1);
                 in = new CachedInputStream(lobIn);
                 lobs.put(lobId, in);
                 lobIn.skip(offset);
@@ -476,47 +519,87 @@ public class TcpServerThread implements Runnable {
             transfer.flush();
             break;
         }
+        case SessionRemote.GET_JDBC_META: {
+            int code = transfer.readInt();
+            int length = transfer.readInt();
+            Value[] args = new Value[length];
+            for (int i = 0; i < length; i++) {
+                args[i] = transfer.readValue(null);
+            }
+            int old = session.getModificationId();
+            ResultInterface result;
+            synchronized (session) {
+                result = DatabaseMetaServer.process(session, code, args);
+            }
+            int columnCount = result.getVisibleColumnCount();
+            int state = getState(old);
+            transfer.writeInt(state).writeInt(columnCount);
+            long rowCount = result.getRowCount();
+            transfer.writeRowCount(rowCount);
+            for (int i = 0; i < columnCount; i++) {
+                ResultColumn.writeColumn(transfer, result, i);
+            }
+            sendRows(result, rowCount);
+            transfer.flush();
+            break;
+        }
         default:
             trace("Unknown operation: " + operation);
-            closeSession();
             close();
         }
     }
 
     private int getState(int oldModificationId) {
+        if (session == null) {
+            return SessionRemote.STATUS_CLOSED;
+        }
         if (session.getModificationId() == oldModificationId) {
-            return SessionRemote.STATUS_OK;
+            long remoteSettingsId = session.getDatabase().getRemoteSettingsId();
+            if (lastRemoteSettingsId == remoteSettingsId) {
+                return SessionRemote.STATUS_OK;
+            }
+            lastRemoteSettingsId = remoteSettingsId;
         }
         return SessionRemote.STATUS_OK_STATE_CHANGED;
     }
 
-    private void sendRow(ResultInterface result) throws IOException {
-        if (result.next()) {
-            transfer.writeBoolean(true);
-            Value[] v = result.currentRow();
-            for (int i = 0; i < result.getVisibleColumnCount(); i++) {
-                if (clientVersion >= Constants.TCP_PROTOCOL_VERSION_12) {
-                    transfer.writeValue(v[i]);
+    private void sendRows(ResultInterface result, long count) throws IOException {
+        int columnCount = result.getVisibleColumnCount();
+        boolean lazy = result.isLazy();
+        Session oldSession = lazy ? session.setThreadLocalSession() : null;
+        try {
+            while (count-- > 0L) {
+                boolean hasNext;
+                try {
+                    hasNext = result.next();
+                } catch (Exception e) {
+                    transfer.writeByte((byte) -1);
+                    sendError(e, false);
+                    break;
+                }
+                if (hasNext) {
+                    transfer.writeByte((byte) 1);
+                    Value[] values = result.currentRow();
+                    for (int i = 0; i < columnCount; i++) {
+                        Value v = values[i];
+                        if (lazy && v instanceof ValueLob) {
+                            ValueLob v2 = ((ValueLob) v).copyToResult();
+                            if (v2 != v) {
+                                v = session.addTemporaryLob(v2);
+                            }
+                        }
+                        transfer.writeValue(v);
+                    }
                 } else {
-                    writeValue(v[i]);
+                    transfer.writeByte((byte) 0);
+                    break;
                 }
             }
-        } else {
-            transfer.writeBoolean(false);
-        }
-    }
-
-    private void writeValue(Value v) throws IOException {
-        if (v.getType() == Value.CLOB || v.getType() == Value.BLOB) {
-            if (v instanceof ValueLobDb) {
-                ValueLobDb lob = (ValueLobDb) v;
-                if (lob.isStored()) {
-                    long id = lob.getLobId();
-                    lobs.put(id, new CachedInputStream(null));
-                }
+        } finally {
+            if (lazy) {
+                session.resetThreadLocalSession(oldSession);
             }
         }
-        transfer.writeValue(v);
     }
 
     void setThread(Thread thread) {
@@ -534,7 +617,7 @@ public class TcpServerThread implements Runnable {
      * @param statementId the statement to cancel
      */
     void cancelStatement(String targetSessionId, int statementId) {
-        if (StringUtils.equals(targetSessionId, this.sessionId)) {
+        if (Objects.equals(targetSessionId, this.sessionId)) {
             Command cmd = (Command) cache.getObject(statementId, false);
             cmd.cancel();
         }
