@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.store;
@@ -9,7 +9,6 @@ import java.io.EOFException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
-import org.h2.util.IOUtils;
 
 /**
  * This class is backed by an input stream and supports reading values and
@@ -32,6 +31,7 @@ public class DataReader extends Reader {
      * Read a byte.
      *
      * @return the byte
+     * @throws IOException on failure
      */
     public byte readByte() throws IOException {
         int x = in.read();
@@ -45,6 +45,7 @@ public class DataReader extends Reader {
      * Read a variable size integer.
      *
      * @return the value
+     * @throws IOException on failure
      */
     public int readVarInt() throws IOException {
         int b = readByte();
@@ -69,76 +70,6 @@ public class DataReader extends Reader {
         return x | ((b & 0x7f) << 21) | (readByte() << 28);
     }
 
-    /**
-     * Read a variable size long.
-     *
-     * @return the value
-     */
-    public long readVarLong() throws IOException {
-        long x = readByte();
-        if (x >= 0) {
-            return x;
-        }
-        x &= 0x7f;
-        for (int s = 7;; s += 7) {
-            long b = readByte();
-            x |= (b & 0x7f) << s;
-            if (b >= 0) {
-                return x;
-            }
-        }
-    }
-
-    /**
-     * Read an integer.
-     *
-     * @return the value
-     */
-    // public int readInt() throws IOException {
-    //     return (read() << 24) + ((read() & 0xff) << 16) +
-    //             ((read() & 0xff) << 8) + (read() & 0xff);
-    //}
-
-    /**
-     * Read a long.
-     *
-     * @return the value
-     */
-    // public long readLong() throws IOException {
-    //    return ((long) (readInt()) << 32) + (readInt() & 0xffffffffL);
-    // }
-
-    /**
-     * Read a number of bytes.
-     *
-     * @param buff the target buffer
-     * @param len the number of bytes to read
-     */
-    public void readFully(byte[] buff, int len) throws IOException {
-        int got = IOUtils.readFully(in, buff, len);
-        if (got < len) {
-            throw new FastEOFException();
-        }
-    }
-
-    /**
-     * Read a string from the stream.
-     *
-     * @return the string
-     */
-    public String readString() throws IOException {
-        int len = readVarInt();
-        return readString(len);
-    }
-
-    private String readString(int len) throws IOException {
-        char[] chars = new char[len];
-        for (int i = 0; i < len; i++) {
-            chars[i] = readChar();
-        }
-        return new String(chars);
-    }
-
     /**
      * Read one character from the input stream.
      *
