@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -1167,7 +1167,10 @@ public class TableFilter implements ColumnResolver {
     public boolean hasInComparisons() {
         for (IndexCondition cond : indexConditions) {
             int compareType = cond.getCompareType();
-            if (compareType == Comparison.IN_QUERY || compareType == Comparison.IN_LIST) {
+            switch (compareType) {
+            case Comparison.IN_LIST:
+            case Comparison.IN_ARRAY:
+            case Comparison.IN_QUERY:
                 return true;
             }
         }
