@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -33,7 +33,12 @@ public enum TableType {
     /**
      * The table type name for external table engines.
      */
-    EXTERNAL_TABLE_ENGINE;
+    EXTERNAL_TABLE_ENGINE,
+
+    /**
+     * The table type name for materialized views.
+     */
+    MATERIALIZED_VIEW;
 
     @Override
     public String toString() {
@@ -43,6 +48,8 @@ public enum TableType {
             return "SYSTEM TABLE";
         } else if (this == TABLE_LINK) {
             return "TABLE LINK";
+        } else if (this == MATERIALIZED_VIEW) {
+            return "MATERIALIZED VIEW";
         } else {
             return super.toString();
         }
