@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -555,7 +555,12 @@ public class Csv implements SimpleRowSource {
                     }
                 }
                 if (i < row.length) {
-                    row[i++] = v;
+                    // Empty Strings should be NULL
+                    // in order to prevent conversion of zero-length String
+                    // to Number
+                    row[i++] = v!=null && v.length() > 0
+                            ? v
+                            : null;
                 }
                 if (endOfLine) {
                     break;
@@ -813,7 +818,7 @@ public class Csv implements SimpleRowSource {
                 continue;
             }
             int index = pair.indexOf('=');
-            String key = StringUtils.trim(pair.substring(0, index), true, true, " ");
+            String key = StringUtils.trimSubstring(pair, 0, index);
             String value = pair.substring(index + 1);
             char ch = value.isEmpty() ? 0 : value.charAt(0);
             if (isParam(key, "escape", "esc", "escapeCharacter")) {
