@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -404,7 +404,7 @@ public class Recover extends Tool implements DataHandler {
     }
 
     private static void dumpLayout(PrintWriter writer, MVStore mv) {
-        MVMap<String, String> layout = mv.getLayoutMap();
+        Map<String, String> layout = mv.getLayoutMap();
         for (Entry<String, String> e : layout.entrySet()) {
             writer.println("-- " + e.getKey() + " = " + e.getValue());
         }
