@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -10,7 +10,7 @@ import java.math.BigDecimal;
 /**
  * JSON number.
  */
-public class JSONNumber extends JSONValue {
+public final class JSONNumber extends JSONValue {
 
     private final BigDecimal value;
 
