@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -21,7 +21,7 @@ public final class ValueArray extends ValueCollectionBase {
 
     private TypeInfo type;
 
-    private TypeInfo componentType;
+    private final TypeInfo componentType;
 
     private ValueArray(TypeInfo componentType, Value[] list, CastDataProvider provider) {
         super(list);
