@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -36,6 +36,7 @@ import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
 import org.h2.store.FileLister;
 import org.h2.store.fs.FileUtils;
+import org.h2.test.TestAll;
 import org.h2.test.TestBase;
 import org.h2.test.TestDb;
 import org.h2.tools.Recover;
@@ -64,8 +65,21 @@ public class TestLob extends TestDb {
      */
     public static void main(String... a) throws Exception {
         TestBase test = TestBase.createCaller().init();
-        test.config.big = true;
-        test.testFromMain();
+        TestAll config = test.config;
+//        config.memory = true;
+//        test.config.big = true;
+//        config.cipher = "AES";
+//        config.cacheType = "SOFT_LRU";
+//        config.diskUndo = true;
+//        config.diskResult = true;
+//        config.traceLevelFile = 1;
+//        config.throttle = 1;
+
+        test.println(config.toString());
+        for (int i = 0; i < 10; i++) {
+            test.testFromMain();
+            test.println("Done pass #" + i);
+        }
     }
 
     @Override
