@@ -1,4 +1,4 @@
--- Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+-- Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
 -- and the EPL 1.0 (https://h2database.com/html/license.html).
 -- Initial Developer: H2 Group
 --
@@ -393,3 +393,192 @@ TABLE TEST;
 
 DROP TABLE TEST;
 > ok
+
+CREATE TABLE TEST(A INT, B INT, C INT, D INT, E INT);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_1 UNIQUE(A, B);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_2 UNIQUE NULLS DISTINCT(A, C);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_3 UNIQUE NULLS ALL DISTINCT(A, D);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_4 UNIQUE NULLS NOT DISTINCT(A, E);
+> ok
+
+SELECT CONSTRAINT_NAME, NULLS_DISTINCT, INDEX_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME = 'TEST';
+> CONSTRAINT_NAME NULLS_DISTINCT INDEX_NAME
+> --------------- -------------- -----------
+> U_1             YES            U_1_INDEX_2
+> U_2             YES            U_2_INDEX_2
+> U_3             ALL            U_3_INDEX_2
+> U_4             NO             U_4_INDEX_2
+> rows: 4
+
+SELECT INDEX_NAME, NULLS_DISTINCT FROM INFORMATION_SCHEMA.INDEXES WHERE TABLE_NAME = 'TEST';
+> INDEX_NAME  NULLS_DISTINCT
+> ----------- --------------
+> U_1_INDEX_2 YES
+> U_2_INDEX_2 YES
+> U_3_INDEX_2 ALL
+> U_4_INDEX_2 NO
+> rows: 4
+
+ALTER TABLE TEST DROP CONSTRAINT U_1;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_2;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_3;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_4;
+> ok
+
+CREATE UNIQUE NULLS DISTINCT INDEX TEST_IDX_1 ON TEST(A, B);
+> ok
+
+CREATE UNIQUE NULLS DISTINCT INDEX TEST_IDX_2 ON TEST(A, C);
+> ok
+
+CREATE UNIQUE NULLS DISTINCT INDEX TEST_IDX_3 ON TEST(A, D);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_1 UNIQUE NULLS DISTINCT(A, B);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_2 UNIQUE NULLS ALL DISTINCT(A, C);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_3 UNIQUE NULLS NOT DISTINCT(A, D);
+> ok
+
+SELECT CONSTRAINT_NAME, INDEX_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME = 'TEST';
+> CONSTRAINT_NAME INDEX_NAME
+> --------------- -----------
+> U_1             TEST_IDX_1
+> U_2             U_2_INDEX_2
+> U_3             U_3_INDEX_2
+> rows: 3
+
+ALTER TABLE TEST DROP CONSTRAINT U_1;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_2;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_3;
+> ok
+
+DROP INDEX TEST_IDX_1;
+> ok
+
+DROP INDEX TEST_IDX_2;
+> ok
+
+DROP INDEX TEST_IDX_3;
+> ok
+
+CREATE UNIQUE NULLS ALL DISTINCT INDEX TEST_IDX_1 ON TEST(A, B);
+> ok
+
+CREATE UNIQUE NULLS ALL DISTINCT INDEX TEST_IDX_2 ON TEST(A, C);
+> ok
+
+CREATE UNIQUE NULLS ALL DISTINCT INDEX TEST_IDX_3 ON TEST(A, D);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_1 UNIQUE NULLS DISTINCT(A, B);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_2 UNIQUE NULLS ALL DISTINCT(A, C);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_3 UNIQUE NULLS NOT DISTINCT(A, D);
+> ok
+
+SELECT CONSTRAINT_NAME, INDEX_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME = 'TEST';
+> CONSTRAINT_NAME INDEX_NAME
+> --------------- -----------
+> U_1             TEST_IDX_1
+> U_2             TEST_IDX_2
+> U_3             U_3_INDEX_2
+> rows: 3
+
+ALTER TABLE TEST DROP CONSTRAINT U_1;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_2;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_3;
+> ok
+
+DROP INDEX TEST_IDX_1;
+> ok
+
+DROP INDEX TEST_IDX_2;
+> ok
+
+DROP INDEX TEST_IDX_3;
+> ok
+
+CREATE UNIQUE NULLS NOT DISTINCT INDEX TEST_IDX_1 ON TEST(A, B);
+> ok
+
+CREATE UNIQUE NULLS NOT DISTINCT INDEX TEST_IDX_2 ON TEST(A, C);
+> ok
+
+CREATE UNIQUE NULLS NOT DISTINCT INDEX TEST_IDX_3 ON TEST(A, D);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_1 UNIQUE NULLS DISTINCT(A, B);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_2 UNIQUE NULLS ALL DISTINCT(A, C);
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_3 UNIQUE NULLS NOT DISTINCT(A, D);
+> ok
+
+SELECT CONSTRAINT_NAME, INDEX_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME = 'TEST';
+> CONSTRAINT_NAME INDEX_NAME
+> --------------- ----------
+> U_1             TEST_IDX_1
+> U_2             TEST_IDX_2
+> U_3             TEST_IDX_3
+> rows: 3
+
+ALTER TABLE TEST DROP CONSTRAINT U_1;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_2;
+> ok
+
+ALTER TABLE TEST DROP CONSTRAINT U_3;
+> ok
+
+DROP INDEX TEST_IDX_1;
+> ok
+
+DROP INDEX TEST_IDX_2;
+> ok
+
+DROP INDEX TEST_IDX_3;
+> ok
+
+ALTER TABLE TEST ADD CONSTRAINT U_4 UNIQUE NULLS ALL DISTINCT(A);
+> ok
+
+SELECT NULLS_DISTINCT FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME = 'TEST';
+>> YES
+
+ALTER TABLE TEST DROP CONSTRAINT U_4;
+> ok
+
+DROP TABLE TEST;
+> ok
