@@ -1,4 +1,4 @@
--- Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+-- Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
 -- and the EPL 1.0 (https://h2database.com/html/license.html).
 -- Initial Developer: H2 Group
 --
@@ -154,3 +154,6 @@ SELECT * FROM V_TEST;
 > 1  val1
 > 2  val2
 > rows: 2
+
+CREATE ALIAS 1;
+> exception SYNTAX_ERROR_2
