@@ -1,4 +1,4 @@
--- Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+-- Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
 -- and the EPL 1.0 (https://h2database.com/html/license.html).
 -- Initial Developer: H2 Group
 --
@@ -216,3 +216,16 @@ EXPLAIN SELECT LISTAGG(V ON OVERFLOW TRUNCATE '..' WITHOUT COUNT) WITHIN GROUP (
 
 DROP TABLE TEST;
 > ok
+
+SELECT LISTAGG(V, ?) L FROM (VALUES 'a', 'b', 'c') T(V);
+{
+:
+> L
+> -----
+> a:b:c
+> rows: 1
+};
+> update count: 0
+
+SELECT LISTAGG(V, V) L FROM (VALUES 'a', 'b', 'c') T(V);
+> exception SYNTAX_ERROR_2
