@@ -1,4 +1,4 @@
--- Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+-- Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
 -- and the EPL 1.0 (https://h2database.com/html/license.html).
 -- Initial Developer: H2 Group
 --
@@ -20,8 +20,14 @@ EXPLAIN SELECT
     BITCOUNT(CAST(X'C5' AS BINARY));
 >> SELECT CAST(4 AS BIGINT), CAST(4 AS BIGINT), CAST(4 AS BIGINT), CAST(4 AS BIGINT), CAST(4 AS BIGINT), CAST(4 AS BIGINT)
 
+SELECT BITCOUNT(X'13');
+>> 3
+
 SELECT BITCOUNT(X'0123456789ABCDEF');
 >> 32
 
-SELECT BITCOUNT(X'0123456789ABCDEF33');
+SELECT BITCOUNT(X'0123456789ABCDEF 33');
 >> 36
+
+SELECT BITCOUNT(X'1111111111111111 3333333333333333 77');
+>> 54
