@@ -1,4 +1,4 @@
--- Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+-- Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
 -- and the EPL 1.0 (https://h2database.com/html/license.html).
 -- Initial Developer: H2 Group
 --
@@ -6,7 +6,7 @@
 select soundex(null) en, soundex('tom') et;
 > EN   ET
 > ---- ----
-> null t500
+> null T500
 > rows: 1
 
 select
