@@ -1,4 +1,4 @@
--- Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+-- Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
 -- and the EPL 1.0 (https://h2database.com/html/license.html).
 -- Initial Developer: H2 Group
 --
@@ -12,6 +12,12 @@ SELECT CARDINALITY(ARRAY[]);
 SELECT CARDINALITY(ARRAY[1, 2, 5]);
 >> 3
 
+SELECT CARDINALITY(JSON '[1, 5]');
+>> 2
+
+SELECT CARDINALITY(JSON 'null');
+>> null
+
 SELECT ARRAY_LENGTH(ARRAY[1, 2, 5]);
 >> 3
 
