@@ -1,4 +1,4 @@
--- Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+-- Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
 -- and the EPL 1.0 (https://h2database.com/html/license.html).
 -- Initial Developer: H2 Group
 --
@@ -201,3 +201,75 @@ SELECT CAST('true ' AS BOOLEAN) V, CAST(CAST('true' AS CHAR(10)) AS BOOLEAN) F;
 > ---- ----
 > TRUE TRUE
 > rows: 1
+
+VALUES CAST(1 AS 1);
+> exception SYNTAX_ERROR_2
+
+SET TIME ZONE 'UTC+10';
+> ok
+
+VALUES CAST(TIME WITH TIME ZONE '10:00:00+01' AS TIME);
+>> 19:00:00
+
+VALUES CAST(TIME WITH TIME ZONE '20:00:00+01' AS TIME);
+>> 05:00:00
+
+VALUES CAST('10:00:00+01' AS TIME);
+>> 19:00:00
+
+VALUES CAST('20:00:00+01' AS TIME);
+>> 05:00:00
+
+SET TIME ZONE LOCAL;
+> ok
+
+VALUES CAST(DATE '2020-05-06' AS VARCHAR FORMAT 'DD.MM.YYYY');
+>> 06.05.2020
+
+VALUES CAST('06.05.2020' AS DATE FORMAT 'DD.MM.YYYY');
+>> 2020-05-06
+
+VALUES CAST(TIME '10:20:30' AS VARCHAR FORMAT 'HH24MISS');
+>> 102030
+
+VALUES CAST('102030' AS TIME FORMAT 'HH24MISS');
+>> 10:20:30
+
+VALUES CAST(TIME WITH TIME ZONE '10:20:30+10:30' AS VARCHAR FORMAT 'HH24MISSTZHTZM');
+>> 102030+1030
+
+VALUES CAST('102030+1030' AS TIME WITH TIME ZONE FORMAT 'HH24MISSTZHTZM');
+>> 10:20:30+10:30
+
+VALUES CAST(TIMESTAMP '2020-05-06 10:20:30' AS VARCHAR FORMAT 'DD.MM.YYYY HH24MISS');
+>> 06.05.2020 102030
+
+VALUES CAST('06.05.2020 102030' AS TIMESTAMP FORMAT 'DD.MM.YYYY HH24MISS');
+>> 2020-05-06 10:20:30
+
+VALUES CAST(TIMESTAMP WITH TIME ZONE '2020-05-06 10:20:30+10:30' AS VARCHAR FORMAT 'DD.MM.YYYY HH24MISSTZHTZM');
+>> 06.05.2020 102030+1030
+
+VALUES CAST('06.05.2020 102030+1030' AS TIMESTAMP WITH TIME ZONE FORMAT 'DD.MM.YYYY HH24MISSTZHTZM');
+>> 2020-05-06 10:20:30+10:30
+
+VALUES CAST(DATE '2023-04-15' AS TIMESTAMP FORMAT 'YYYY-MM-DD');
+> exception FEATURE_NOT_SUPPORTED_1
+
+VALUES CAST('AA' AS VARCHAR(100) FORMAT 'YYYY-MM-DD');
+> exception FEATURE_NOT_SUPPORTED_1
+
+VALUES CAST(DATE '2023-04-15' AS VARCHAR FORMAT 'YYYY-MM-DD HH24');
+> exception PARSE_ERROR_1
+
+SELECT CAST(D AS VARCHAR FORMAT F) FROM
+(VALUES (DATE '1990-05-18', 'YYYY-MM-DD'), (DATE '2000-06-30', 'DD-MM-YYYY'), (CURRENT_DATE, NULL)) T(D, F);
+> CAST(D AS CHARACTER VARYING FORMAT F)
+> -------------------------------------
+> 1990-05-18
+> 30-06-2000
+> null
+> rows: 3
+
+SELECT 1::BIGINT::NUMERIC;
+>> 1
