@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -141,8 +141,8 @@ public class TestRandomMapOps extends TestBase {
             case 10:
                 log(op, k, v, "s.commit()");
                 s.commit();
-                log(op, k, v, "s.compactMoveChunks()");
-                s.compactMoveChunks();
+                log(op, k, v, "s.compactFile(0)");
+                s.compactFile(0);
                 break;
             case 11: {
                 int rangeSize = r.nextInt(2 * keysPerPage);
