@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -132,6 +132,10 @@ public class BnfRandom implements BnfVisitor {
         }
         case RuleFixed.HEX_START:
             return "0x";
+        case RuleFixed.OCTAL_START:
+            return "0b";
+        case RuleFixed.BINARY_START:
+            return "0b";
         case RuleFixed.CONCAT:
             return "||";
         case RuleFixed.AZ_UNDERSCORE:
