@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2023 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -87,12 +87,18 @@ public class TestFileLock extends TestDb implements Runnable {
     }
 
     private void testSimple() {
-        FileLock lock1 = new FileLock(new TraceSystem(null), getFile(), Constants.LOCK_SLEEP);
-        FileLock lock2 = new FileLock(new TraceSystem(null), getFile(), Constants.LOCK_SLEEP);
+        String fileName = getFile();
+        testSimple(fileName);
+        testSimple("async:" + fileName);
+    }
+
+    private void testSimple(String fileName) {
+        FileLock lock1 = new FileLock(new TraceSystem(null), fileName, Constants.LOCK_SLEEP);
+        FileLock lock2 = new FileLock(new TraceSystem(null), fileName, Constants.LOCK_SLEEP);
         lock1.lock(FileLockMethod.FILE);
         assertThrows(ErrorCode.DATABASE_ALREADY_OPEN_1, () -> lock2.lock(FileLockMethod.FILE));
         lock1.unlock();
-        FileLock lock3 = new FileLock(new TraceSystem(null), getFile(), Constants.LOCK_SLEEP);
+        FileLock lock3 = new FileLock(new TraceSystem(null), fileName, Constants.LOCK_SLEEP);
         lock3.lock(FileLockMethod.FILE);
         lock3.unlock();
     }
